# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['TrafficMarkingPolicyArgs', 'TrafficMarkingPolicy']

@pulumi.input_type
class TrafficMarkingPolicyArgs:
    def __init__(__self__, *,
                 marking_dscp: pulumi.Input[int],
                 priority: pulumi.Input[int],
                 transit_router_id: pulumi.Input[str],
                 description: Optional[pulumi.Input[str]] = None,
                 dry_run: Optional[pulumi.Input[bool]] = None,
                 traffic_marking_policy_name: Optional[pulumi.Input[str]] = None,
                 traffic_match_rules: Optional[pulumi.Input[Sequence[pulumi.Input['TrafficMarkingPolicyTrafficMatchRuleArgs']]]] = None):
        """
        The set of arguments for constructing a TrafficMarkingPolicy resource.
        :param pulumi.Input[int] marking_dscp: MarkingDscp
        :param pulumi.Input[int] priority: Priority
        :param pulumi.Input[str] transit_router_id: TransitRouterId
        :param pulumi.Input[str] description: TrafficMarkingPolicyDescription
        :param pulumi.Input[bool] dry_run: Whether to PreCheck only this request. Value:
        :param pulumi.Input[str] traffic_marking_policy_name: TrafficMarkingPolicyName
        :param pulumi.Input[Sequence[pulumi.Input['TrafficMarkingPolicyTrafficMatchRuleArgs']]] traffic_match_rules: List of stream classification rules.
               
               You can add up to 50 stream classification rules at a time. See `traffic_match_rules` below.
        """
        pulumi.set(__self__, "marking_dscp", marking_dscp)
        pulumi.set(__self__, "priority", priority)
        pulumi.set(__self__, "transit_router_id", transit_router_id)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if dry_run is not None:
            pulumi.set(__self__, "dry_run", dry_run)
        if traffic_marking_policy_name is not None:
            pulumi.set(__self__, "traffic_marking_policy_name", traffic_marking_policy_name)
        if traffic_match_rules is not None:
            pulumi.set(__self__, "traffic_match_rules", traffic_match_rules)

    @property
    @pulumi.getter(name="markingDscp")
    def marking_dscp(self) -> pulumi.Input[int]:
        """
        MarkingDscp
        """
        return pulumi.get(self, "marking_dscp")

    @marking_dscp.setter
    def marking_dscp(self, value: pulumi.Input[int]):
        pulumi.set(self, "marking_dscp", value)

    @property
    @pulumi.getter
    def priority(self) -> pulumi.Input[int]:
        """
        Priority
        """
        return pulumi.get(self, "priority")

    @priority.setter
    def priority(self, value: pulumi.Input[int]):
        pulumi.set(self, "priority", value)

    @property
    @pulumi.getter(name="transitRouterId")
    def transit_router_id(self) -> pulumi.Input[str]:
        """
        TransitRouterId
        """
        return pulumi.get(self, "transit_router_id")

    @transit_router_id.setter
    def transit_router_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "transit_router_id", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        TrafficMarkingPolicyDescription
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="dryRun")
    def dry_run(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to PreCheck only this request. Value:
        """
        return pulumi.get(self, "dry_run")

    @dry_run.setter
    def dry_run(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "dry_run", value)

    @property
    @pulumi.getter(name="trafficMarkingPolicyName")
    def traffic_marking_policy_name(self) -> Optional[pulumi.Input[str]]:
        """
        TrafficMarkingPolicyName
        """
        return pulumi.get(self, "traffic_marking_policy_name")

    @traffic_marking_policy_name.setter
    def traffic_marking_policy_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "traffic_marking_policy_name", value)

    @property
    @pulumi.getter(name="trafficMatchRules")
    def traffic_match_rules(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['TrafficMarkingPolicyTrafficMatchRuleArgs']]]]:
        """
        List of stream classification rules.

        You can add up to 50 stream classification rules at a time. See `traffic_match_rules` below.
        """
        return pulumi.get(self, "traffic_match_rules")

    @traffic_match_rules.setter
    def traffic_match_rules(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['TrafficMarkingPolicyTrafficMatchRuleArgs']]]]):
        pulumi.set(self, "traffic_match_rules", value)


@pulumi.input_type
class _TrafficMarkingPolicyState:
    def __init__(__self__, *,
                 description: Optional[pulumi.Input[str]] = None,
                 dry_run: Optional[pulumi.Input[bool]] = None,
                 marking_dscp: Optional[pulumi.Input[int]] = None,
                 priority: Optional[pulumi.Input[int]] = None,
                 status: Optional[pulumi.Input[str]] = None,
                 traffic_marking_policy_id: Optional[pulumi.Input[str]] = None,
                 traffic_marking_policy_name: Optional[pulumi.Input[str]] = None,
                 traffic_match_rules: Optional[pulumi.Input[Sequence[pulumi.Input['TrafficMarkingPolicyTrafficMatchRuleArgs']]]] = None,
                 transit_router_id: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering TrafficMarkingPolicy resources.
        :param pulumi.Input[str] description: TrafficMarkingPolicyDescription
        :param pulumi.Input[bool] dry_run: Whether to PreCheck only this request. Value:
        :param pulumi.Input[int] marking_dscp: MarkingDscp
        :param pulumi.Input[int] priority: Priority
        :param pulumi.Input[str] status: The status of the resource
        :param pulumi.Input[str] traffic_marking_policy_id: The first ID of the resource
        :param pulumi.Input[str] traffic_marking_policy_name: TrafficMarkingPolicyName
        :param pulumi.Input[Sequence[pulumi.Input['TrafficMarkingPolicyTrafficMatchRuleArgs']]] traffic_match_rules: List of stream classification rules.
               
               You can add up to 50 stream classification rules at a time. See `traffic_match_rules` below.
        :param pulumi.Input[str] transit_router_id: TransitRouterId
        """
        if description is not None:
            pulumi.set(__self__, "description", description)
        if dry_run is not None:
            pulumi.set(__self__, "dry_run", dry_run)
        if marking_dscp is not None:
            pulumi.set(__self__, "marking_dscp", marking_dscp)
        if priority is not None:
            pulumi.set(__self__, "priority", priority)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if traffic_marking_policy_id is not None:
            pulumi.set(__self__, "traffic_marking_policy_id", traffic_marking_policy_id)
        if traffic_marking_policy_name is not None:
            pulumi.set(__self__, "traffic_marking_policy_name", traffic_marking_policy_name)
        if traffic_match_rules is not None:
            pulumi.set(__self__, "traffic_match_rules", traffic_match_rules)
        if transit_router_id is not None:
            pulumi.set(__self__, "transit_router_id", transit_router_id)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        TrafficMarkingPolicyDescription
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="dryRun")
    def dry_run(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to PreCheck only this request. Value:
        """
        return pulumi.get(self, "dry_run")

    @dry_run.setter
    def dry_run(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "dry_run", value)

    @property
    @pulumi.getter(name="markingDscp")
    def marking_dscp(self) -> Optional[pulumi.Input[int]]:
        """
        MarkingDscp
        """
        return pulumi.get(self, "marking_dscp")

    @marking_dscp.setter
    def marking_dscp(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "marking_dscp", value)

    @property
    @pulumi.getter
    def priority(self) -> Optional[pulumi.Input[int]]:
        """
        Priority
        """
        return pulumi.get(self, "priority")

    @priority.setter
    def priority(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "priority", value)

    @property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[str]]:
        """
        The status of the resource
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "status", value)

    @property
    @pulumi.getter(name="trafficMarkingPolicyId")
    def traffic_marking_policy_id(self) -> Optional[pulumi.Input[str]]:
        """
        The first ID of the resource
        """
        return pulumi.get(self, "traffic_marking_policy_id")

    @traffic_marking_policy_id.setter
    def traffic_marking_policy_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "traffic_marking_policy_id", value)

    @property
    @pulumi.getter(name="trafficMarkingPolicyName")
    def traffic_marking_policy_name(self) -> Optional[pulumi.Input[str]]:
        """
        TrafficMarkingPolicyName
        """
        return pulumi.get(self, "traffic_marking_policy_name")

    @traffic_marking_policy_name.setter
    def traffic_marking_policy_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "traffic_marking_policy_name", value)

    @property
    @pulumi.getter(name="trafficMatchRules")
    def traffic_match_rules(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['TrafficMarkingPolicyTrafficMatchRuleArgs']]]]:
        """
        List of stream classification rules.

        You can add up to 50 stream classification rules at a time. See `traffic_match_rules` below.
        """
        return pulumi.get(self, "traffic_match_rules")

    @traffic_match_rules.setter
    def traffic_match_rules(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['TrafficMarkingPolicyTrafficMatchRuleArgs']]]]):
        pulumi.set(self, "traffic_match_rules", value)

    @property
    @pulumi.getter(name="transitRouterId")
    def transit_router_id(self) -> Optional[pulumi.Input[str]]:
        """
        TransitRouterId
        """
        return pulumi.get(self, "transit_router_id")

    @transit_router_id.setter
    def transit_router_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "transit_router_id", value)


class TrafficMarkingPolicy(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 dry_run: Optional[pulumi.Input[bool]] = None,
                 marking_dscp: Optional[pulumi.Input[int]] = None,
                 priority: Optional[pulumi.Input[int]] = None,
                 traffic_marking_policy_name: Optional[pulumi.Input[str]] = None,
                 traffic_match_rules: Optional[pulumi.Input[Sequence[pulumi.Input[Union['TrafficMarkingPolicyTrafficMatchRuleArgs', 'TrafficMarkingPolicyTrafficMatchRuleArgsDict']]]]] = None,
                 transit_router_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Provides a Cloud Enterprise Network (CEN) Traffic Marking Policy resource.

        For information about Cloud Enterprise Network (CEN) Traffic Marking Policy and how to use it, see [What is Traffic Marking Policy](https://www.alibabacloud.com/help/en/cen/developer-reference/api-cbn-2017-09-12-createtrafficmarkingpolicy).

        > **NOTE:** Available since v1.173.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        example = alicloud.cen.Instance("example",
            cen_instance_name="tf_example",
            description="an example for cen")
        example_transit_router = alicloud.cen.TransitRouter("example",
            transit_router_name="tf_example",
            cen_id=example.id)
        example_traffic_marking_policy = alicloud.cen.TrafficMarkingPolicy("example",
            marking_dscp=1,
            priority=1,
            traffic_marking_policy_name="tf_example",
            transit_router_id=example_transit_router.transit_router_id)
        ```

        ## Import

        CEN Traffic Marking Policy can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:cen/trafficMarkingPolicy:TrafficMarkingPolicy example <transit_router_id>:<traffic_marking_policy_id>
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: TrafficMarkingPolicyDescription
        :param pulumi.Input[bool] dry_run: Whether to PreCheck only this request. Value:
        :param pulumi.Input[int] marking_dscp: MarkingDscp
        :param pulumi.Input[int] priority: Priority
        :param pulumi.Input[str] traffic_marking_policy_name: TrafficMarkingPolicyName
        :param pulumi.Input[Sequence[pulumi.Input[Union['TrafficMarkingPolicyTrafficMatchRuleArgs', 'TrafficMarkingPolicyTrafficMatchRuleArgsDict']]]] traffic_match_rules: List of stream classification rules.
               
               You can add up to 50 stream classification rules at a time. See `traffic_match_rules` below.
        :param pulumi.Input[str] transit_router_id: TransitRouterId
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: TrafficMarkingPolicyArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a Cloud Enterprise Network (CEN) Traffic Marking Policy resource.

        For information about Cloud Enterprise Network (CEN) Traffic Marking Policy and how to use it, see [What is Traffic Marking Policy](https://www.alibabacloud.com/help/en/cen/developer-reference/api-cbn-2017-09-12-createtrafficmarkingpolicy).

        > **NOTE:** Available since v1.173.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        example = alicloud.cen.Instance("example",
            cen_instance_name="tf_example",
            description="an example for cen")
        example_transit_router = alicloud.cen.TransitRouter("example",
            transit_router_name="tf_example",
            cen_id=example.id)
        example_traffic_marking_policy = alicloud.cen.TrafficMarkingPolicy("example",
            marking_dscp=1,
            priority=1,
            traffic_marking_policy_name="tf_example",
            transit_router_id=example_transit_router.transit_router_id)
        ```

        ## Import

        CEN Traffic Marking Policy can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:cen/trafficMarkingPolicy:TrafficMarkingPolicy example <transit_router_id>:<traffic_marking_policy_id>
        ```

        :param str resource_name: The name of the resource.
        :param TrafficMarkingPolicyArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(TrafficMarkingPolicyArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 dry_run: Optional[pulumi.Input[bool]] = None,
                 marking_dscp: Optional[pulumi.Input[int]] = None,
                 priority: Optional[pulumi.Input[int]] = None,
                 traffic_marking_policy_name: Optional[pulumi.Input[str]] = None,
                 traffic_match_rules: Optional[pulumi.Input[Sequence[pulumi.Input[Union['TrafficMarkingPolicyTrafficMatchRuleArgs', 'TrafficMarkingPolicyTrafficMatchRuleArgsDict']]]]] = None,
                 transit_router_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = TrafficMarkingPolicyArgs.__new__(TrafficMarkingPolicyArgs)

            __props__.__dict__["description"] = description
            __props__.__dict__["dry_run"] = dry_run
            if marking_dscp is None and not opts.urn:
                raise TypeError("Missing required property 'marking_dscp'")
            __props__.__dict__["marking_dscp"] = marking_dscp
            if priority is None and not opts.urn:
                raise TypeError("Missing required property 'priority'")
            __props__.__dict__["priority"] = priority
            __props__.__dict__["traffic_marking_policy_name"] = traffic_marking_policy_name
            __props__.__dict__["traffic_match_rules"] = traffic_match_rules
            if transit_router_id is None and not opts.urn:
                raise TypeError("Missing required property 'transit_router_id'")
            __props__.__dict__["transit_router_id"] = transit_router_id
            __props__.__dict__["status"] = None
            __props__.__dict__["traffic_marking_policy_id"] = None
        super(TrafficMarkingPolicy, __self__).__init__(
            'alicloud:cen/trafficMarkingPolicy:TrafficMarkingPolicy',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            description: Optional[pulumi.Input[str]] = None,
            dry_run: Optional[pulumi.Input[bool]] = None,
            marking_dscp: Optional[pulumi.Input[int]] = None,
            priority: Optional[pulumi.Input[int]] = None,
            status: Optional[pulumi.Input[str]] = None,
            traffic_marking_policy_id: Optional[pulumi.Input[str]] = None,
            traffic_marking_policy_name: Optional[pulumi.Input[str]] = None,
            traffic_match_rules: Optional[pulumi.Input[Sequence[pulumi.Input[Union['TrafficMarkingPolicyTrafficMatchRuleArgs', 'TrafficMarkingPolicyTrafficMatchRuleArgsDict']]]]] = None,
            transit_router_id: Optional[pulumi.Input[str]] = None) -> 'TrafficMarkingPolicy':
        """
        Get an existing TrafficMarkingPolicy resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: TrafficMarkingPolicyDescription
        :param pulumi.Input[bool] dry_run: Whether to PreCheck only this request. Value:
        :param pulumi.Input[int] marking_dscp: MarkingDscp
        :param pulumi.Input[int] priority: Priority
        :param pulumi.Input[str] status: The status of the resource
        :param pulumi.Input[str] traffic_marking_policy_id: The first ID of the resource
        :param pulumi.Input[str] traffic_marking_policy_name: TrafficMarkingPolicyName
        :param pulumi.Input[Sequence[pulumi.Input[Union['TrafficMarkingPolicyTrafficMatchRuleArgs', 'TrafficMarkingPolicyTrafficMatchRuleArgsDict']]]] traffic_match_rules: List of stream classification rules.
               
               You can add up to 50 stream classification rules at a time. See `traffic_match_rules` below.
        :param pulumi.Input[str] transit_router_id: TransitRouterId
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _TrafficMarkingPolicyState.__new__(_TrafficMarkingPolicyState)

        __props__.__dict__["description"] = description
        __props__.__dict__["dry_run"] = dry_run
        __props__.__dict__["marking_dscp"] = marking_dscp
        __props__.__dict__["priority"] = priority
        __props__.__dict__["status"] = status
        __props__.__dict__["traffic_marking_policy_id"] = traffic_marking_policy_id
        __props__.__dict__["traffic_marking_policy_name"] = traffic_marking_policy_name
        __props__.__dict__["traffic_match_rules"] = traffic_match_rules
        __props__.__dict__["transit_router_id"] = transit_router_id
        return TrafficMarkingPolicy(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[str]]:
        """
        TrafficMarkingPolicyDescription
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="dryRun")
    def dry_run(self) -> pulumi.Output[Optional[bool]]:
        """
        Whether to PreCheck only this request. Value:
        """
        return pulumi.get(self, "dry_run")

    @property
    @pulumi.getter(name="markingDscp")
    def marking_dscp(self) -> pulumi.Output[int]:
        """
        MarkingDscp
        """
        return pulumi.get(self, "marking_dscp")

    @property
    @pulumi.getter
    def priority(self) -> pulumi.Output[int]:
        """
        Priority
        """
        return pulumi.get(self, "priority")

    @property
    @pulumi.getter
    def status(self) -> pulumi.Output[str]:
        """
        The status of the resource
        """
        return pulumi.get(self, "status")

    @property
    @pulumi.getter(name="trafficMarkingPolicyId")
    def traffic_marking_policy_id(self) -> pulumi.Output[str]:
        """
        The first ID of the resource
        """
        return pulumi.get(self, "traffic_marking_policy_id")

    @property
    @pulumi.getter(name="trafficMarkingPolicyName")
    def traffic_marking_policy_name(self) -> pulumi.Output[Optional[str]]:
        """
        TrafficMarkingPolicyName
        """
        return pulumi.get(self, "traffic_marking_policy_name")

    @property
    @pulumi.getter(name="trafficMatchRules")
    def traffic_match_rules(self) -> pulumi.Output[Optional[Sequence['outputs.TrafficMarkingPolicyTrafficMatchRule']]]:
        """
        List of stream classification rules.

        You can add up to 50 stream classification rules at a time. See `traffic_match_rules` below.
        """
        return pulumi.get(self, "traffic_match_rules")

    @property
    @pulumi.getter(name="transitRouterId")
    def transit_router_id(self) -> pulumi.Output[str]:
        """
        TransitRouterId
        """
        return pulumi.get(self, "transit_router_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = ['NatFirewallControlPolicyArgs', 'NatFirewallControlPolicy']

@pulumi.input_type
class NatFirewallControlPolicyArgs:
    def __init__(__self__, *,
                 acl_action: pulumi.Input[str],
                 application_name_lists: pulumi.Input[Sequence[pulumi.Input[str]]],
                 description: pulumi.Input[str],
                 destination: pulumi.Input[str],
                 destination_type: pulumi.Input[str],
                 direction: pulumi.Input[str],
                 nat_gateway_id: pulumi.Input[str],
                 new_order: pulumi.Input[str],
                 proto: pulumi.Input[str],
                 source: pulumi.Input[str],
                 source_type: pulumi.Input[str],
                 dest_port: Optional[pulumi.Input[str]] = None,
                 dest_port_group: Optional[pulumi.Input[str]] = None,
                 dest_port_type: Optional[pulumi.Input[str]] = None,
                 domain_resolve_type: Optional[pulumi.Input[int]] = None,
                 end_time: Optional[pulumi.Input[int]] = None,
                 ip_version: Optional[pulumi.Input[str]] = None,
                 release: Optional[pulumi.Input[str]] = None,
                 repeat_days: Optional[pulumi.Input[Sequence[pulumi.Input[int]]]] = None,
                 repeat_end_time: Optional[pulumi.Input[str]] = None,
                 repeat_start_time: Optional[pulumi.Input[str]] = None,
                 repeat_type: Optional[pulumi.Input[str]] = None,
                 start_time: Optional[pulumi.Input[int]] = None):
        """
        The set of arguments for constructing a NatFirewallControlPolicy resource.
        :param pulumi.Input[str] acl_action: The method (action) of access traffic passing through Cloud Firewall in the security access control policy. Valid values:
               - **accept**: Release
               - **drop**: Refused
               - **log**: Observation.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] application_name_lists: The list of application types supported by the access control policy.
        :param pulumi.Input[str] description: The description of the access control policy.
        :param pulumi.Input[str] destination: The destination address segment in the access control policy. Valid values:
               - When DestinationType is net, Destination is the Destination CIDR. For example: 1.2.XX.XX/24
               - When DestinationType IS group, Destination is the name of the Destination address book. For example: db_group
               - When DestinationType is domain, Destination is the Destination domain name. For example: * .aliyuncs.com
               - When DestinationType is location, Destination is the Destination region. For example: \\["BJ11", "ZB"\\].
        :param pulumi.Input[str] destination_type: The destination address type in the access control policy. Valid values:
               - **net**: Destination Network segment (CIDR address)
               - **group**: Destination Address Book
               - **domain**: the destination domain name.
        :param pulumi.Input[str] direction: The traffic direction of the access control policy. Valid values:
               - **out**: Internal and external traffic access control.
        :param pulumi.Input[str] nat_gateway_id: The ID of the NAT gateway instance.
        :param pulumi.Input[str] new_order: The priority for the access control policy to take effect. The priority number increases sequentially from 1, and the smaller the priority number, the higher the priority.
        :param pulumi.Input[str] proto: The security protocol type for traffic access in the access control policy. Valid values:
               - ANY (indicates that all protocol types are queried)
               - TCP
               - UDP
               - ICMP.
        :param pulumi.Input[str] source: The source address in the access control policy. Valid values:
               - When **SourceType** is set to 'net', Source is the Source CIDR address. For example: 10.2.4.0/24
               - When **SourceType** is set to 'group', Source is the name of the Source address book. For example: db_group.
        :param pulumi.Input[str] source_type: The source address type in the access control policy. Valid values:
               - **net**: the source network segment (CIDR address)
               - **group**: source address book
        :param pulumi.Input[str] dest_port: The destination port of traffic access in the access control policy. Value:
               - When the protocol type is set to ICMP, the value of DestPort is null.
               > **NOTE:**  When the protocol type is ICMP, access control on the destination port is not supported.
               - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) IS group, the value of DestPort is null.
               > **NOTE:**  When you select group (destination port address book) for the destination port type of the access control policy, you do not need to set a specific destination port number. All ports that need to be controlled by this access control policy are included in the destination port address book.
               - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) is port, the value of DestPort is the destination port number.
        :param pulumi.Input[str] dest_port_group: The address book name of the destination port of the access traffic in the access control policy.
               > **NOTE:**  When DestPortType is set to group, you need to set the destination port address book name.
        :param pulumi.Input[str] dest_port_type: The destination port type of the access traffic in the security access control policy.
               - **port**: port
               - **group**: Port Address Book.
        :param pulumi.Input[int] domain_resolve_type: The domain name resolution method of the access control policy. The policy is enabled by default after it is created. Valid values:
               - **0**: Based on FQDN
               - **1**: DNS-based dynamic resolution
               - **2**: dynamic resolution based on FQDN and DNS.
        :param pulumi.Input[int] end_time: The end time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. Must be full or half time and at least half an hour greater than the start time.
               > **NOTE:**  When RepeatType is set to permit, EndTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, EndTime must have a value and you need to set the end time.
        :param pulumi.Input[str] ip_version: Supported IP address version. Value:
               - **4** (default): indicates the IPv4 address.
        :param pulumi.Input[str] release: The enabled status of the access control policy. The policy is enabled by default after it is created. Value:
               - **true**: Enable access control policy
               - **false**: Do not enable access control policies.
        :param pulumi.Input[Sequence[pulumi.Input[int]]] repeat_days: Collection of recurring dates for the policy validity period of the access control policy.
               - When RepeatType is 'Permanent', 'None', 'Daily', RepeatDays is an empty collection. For example:[]
               - When RepeatType is Weekly, RepeatDays cannot be empty. For example:["0", "6"]. When the RepeatType is set to Weekly, RepeatDays cannot be repeated.
               - RepeatDays cannot be empty when RepeatType is 'Monthly. For example:[1, 31]. When RepeatType is set to Monthly, RepeatDays cannot be repeated.
        :param pulumi.Input[str] repeat_end_time: The recurring end time of the policy validity period of the access control policy. For example: 23:30, it must be the whole point or half point time, and at least half an hour greater than the repeat start time.
               > **NOTE:**  When RepeatType is set to normal or None, RepeatEndTime is null. When the RepeatType is Daily, Weekly, or Monthly, the RepeatEndTime must have a value, and you need to set the repeat end time.
        :param pulumi.Input[str] repeat_start_time: The recurring start time of the policy validity period of the access control policy. For example: 08:00, it must be the whole point or half point time, and at least half an hour less than the repeat end time.
               > **NOTE:**  When RepeatType is set to permit or None, RepeatStartTime is empty. When the RepeatType is Daily, Weekly, or Monthly, the RepeatStartTime must have a value and you need to set the repeat start time.
        :param pulumi.Input[str] repeat_type: The type of repetition for the policy validity period of the access control policy. Value:
               - **Permit** (default): Always
               - **None**: Specify a single time
               - **Daily**: Daily
               - **Weekly**: Weekly
               - **Monthly**: Monthly.
        :param pulumi.Input[int] start_time: The start time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. It must be a full or half hour and at least half an hour less than the end time.
               > **NOTE:**  When RepeatType is set to normal, StartTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, StartTime must have a value and you need to set the start time.
        """
        pulumi.set(__self__, "acl_action", acl_action)
        pulumi.set(__self__, "application_name_lists", application_name_lists)
        pulumi.set(__self__, "description", description)
        pulumi.set(__self__, "destination", destination)
        pulumi.set(__self__, "destination_type", destination_type)
        pulumi.set(__self__, "direction", direction)
        pulumi.set(__self__, "nat_gateway_id", nat_gateway_id)
        pulumi.set(__self__, "new_order", new_order)
        pulumi.set(__self__, "proto", proto)
        pulumi.set(__self__, "source", source)
        pulumi.set(__self__, "source_type", source_type)
        if dest_port is not None:
            pulumi.set(__self__, "dest_port", dest_port)
        if dest_port_group is not None:
            pulumi.set(__self__, "dest_port_group", dest_port_group)
        if dest_port_type is not None:
            pulumi.set(__self__, "dest_port_type", dest_port_type)
        if domain_resolve_type is not None:
            pulumi.set(__self__, "domain_resolve_type", domain_resolve_type)
        if end_time is not None:
            pulumi.set(__self__, "end_time", end_time)
        if ip_version is not None:
            pulumi.set(__self__, "ip_version", ip_version)
        if release is not None:
            pulumi.set(__self__, "release", release)
        if repeat_days is not None:
            pulumi.set(__self__, "repeat_days", repeat_days)
        if repeat_end_time is not None:
            pulumi.set(__self__, "repeat_end_time", repeat_end_time)
        if repeat_start_time is not None:
            pulumi.set(__self__, "repeat_start_time", repeat_start_time)
        if repeat_type is not None:
            pulumi.set(__self__, "repeat_type", repeat_type)
        if start_time is not None:
            pulumi.set(__self__, "start_time", start_time)

    @property
    @pulumi.getter(name="aclAction")
    def acl_action(self) -> pulumi.Input[str]:
        """
        The method (action) of access traffic passing through Cloud Firewall in the security access control policy. Valid values:
        - **accept**: Release
        - **drop**: Refused
        - **log**: Observation.
        """
        return pulumi.get(self, "acl_action")

    @acl_action.setter
    def acl_action(self, value: pulumi.Input[str]):
        pulumi.set(self, "acl_action", value)

    @property
    @pulumi.getter(name="applicationNameLists")
    def application_name_lists(self) -> pulumi.Input[Sequence[pulumi.Input[str]]]:
        """
        The list of application types supported by the access control policy.
        """
        return pulumi.get(self, "application_name_lists")

    @application_name_lists.setter
    def application_name_lists(self, value: pulumi.Input[Sequence[pulumi.Input[str]]]):
        pulumi.set(self, "application_name_lists", value)

    @property
    @pulumi.getter
    def description(self) -> pulumi.Input[str]:
        """
        The description of the access control policy.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: pulumi.Input[str]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter
    def destination(self) -> pulumi.Input[str]:
        """
        The destination address segment in the access control policy. Valid values:
        - When DestinationType is net, Destination is the Destination CIDR. For example: 1.2.XX.XX/24
        - When DestinationType IS group, Destination is the name of the Destination address book. For example: db_group
        - When DestinationType is domain, Destination is the Destination domain name. For example: * .aliyuncs.com
        - When DestinationType is location, Destination is the Destination region. For example: \\["BJ11", "ZB"\\].
        """
        return pulumi.get(self, "destination")

    @destination.setter
    def destination(self, value: pulumi.Input[str]):
        pulumi.set(self, "destination", value)

    @property
    @pulumi.getter(name="destinationType")
    def destination_type(self) -> pulumi.Input[str]:
        """
        The destination address type in the access control policy. Valid values:
        - **net**: Destination Network segment (CIDR address)
        - **group**: Destination Address Book
        - **domain**: the destination domain name.
        """
        return pulumi.get(self, "destination_type")

    @destination_type.setter
    def destination_type(self, value: pulumi.Input[str]):
        pulumi.set(self, "destination_type", value)

    @property
    @pulumi.getter
    def direction(self) -> pulumi.Input[str]:
        """
        The traffic direction of the access control policy. Valid values:
        - **out**: Internal and external traffic access control.
        """
        return pulumi.get(self, "direction")

    @direction.setter
    def direction(self, value: pulumi.Input[str]):
        pulumi.set(self, "direction", value)

    @property
    @pulumi.getter(name="natGatewayId")
    def nat_gateway_id(self) -> pulumi.Input[str]:
        """
        The ID of the NAT gateway instance.
        """
        return pulumi.get(self, "nat_gateway_id")

    @nat_gateway_id.setter
    def nat_gateway_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "nat_gateway_id", value)

    @property
    @pulumi.getter(name="newOrder")
    def new_order(self) -> pulumi.Input[str]:
        """
        The priority for the access control policy to take effect. The priority number increases sequentially from 1, and the smaller the priority number, the higher the priority.
        """
        return pulumi.get(self, "new_order")

    @new_order.setter
    def new_order(self, value: pulumi.Input[str]):
        pulumi.set(self, "new_order", value)

    @property
    @pulumi.getter
    def proto(self) -> pulumi.Input[str]:
        """
        The security protocol type for traffic access in the access control policy. Valid values:
        - ANY (indicates that all protocol types are queried)
        - TCP
        - UDP
        - ICMP.
        """
        return pulumi.get(self, "proto")

    @proto.setter
    def proto(self, value: pulumi.Input[str]):
        pulumi.set(self, "proto", value)

    @property
    @pulumi.getter
    def source(self) -> pulumi.Input[str]:
        """
        The source address in the access control policy. Valid values:
        - When **SourceType** is set to 'net', Source is the Source CIDR address. For example: 10.2.4.0/24
        - When **SourceType** is set to 'group', Source is the name of the Source address book. For example: db_group.
        """
        return pulumi.get(self, "source")

    @source.setter
    def source(self, value: pulumi.Input[str]):
        pulumi.set(self, "source", value)

    @property
    @pulumi.getter(name="sourceType")
    def source_type(self) -> pulumi.Input[str]:
        """
        The source address type in the access control policy. Valid values:
        - **net**: the source network segment (CIDR address)
        - **group**: source address book
        """
        return pulumi.get(self, "source_type")

    @source_type.setter
    def source_type(self, value: pulumi.Input[str]):
        pulumi.set(self, "source_type", value)

    @property
    @pulumi.getter(name="destPort")
    def dest_port(self) -> Optional[pulumi.Input[str]]:
        """
        The destination port of traffic access in the access control policy. Value:
        - When the protocol type is set to ICMP, the value of DestPort is null.
        > **NOTE:**  When the protocol type is ICMP, access control on the destination port is not supported.
        - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) IS group, the value of DestPort is null.
        > **NOTE:**  When you select group (destination port address book) for the destination port type of the access control policy, you do not need to set a specific destination port number. All ports that need to be controlled by this access control policy are included in the destination port address book.
        - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) is port, the value of DestPort is the destination port number.
        """
        return pulumi.get(self, "dest_port")

    @dest_port.setter
    def dest_port(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "dest_port", value)

    @property
    @pulumi.getter(name="destPortGroup")
    def dest_port_group(self) -> Optional[pulumi.Input[str]]:
        """
        The address book name of the destination port of the access traffic in the access control policy.
        > **NOTE:**  When DestPortType is set to group, you need to set the destination port address book name.
        """
        return pulumi.get(self, "dest_port_group")

    @dest_port_group.setter
    def dest_port_group(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "dest_port_group", value)

    @property
    @pulumi.getter(name="destPortType")
    def dest_port_type(self) -> Optional[pulumi.Input[str]]:
        """
        The destination port type of the access traffic in the security access control policy.
        - **port**: port
        - **group**: Port Address Book.
        """
        return pulumi.get(self, "dest_port_type")

    @dest_port_type.setter
    def dest_port_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "dest_port_type", value)

    @property
    @pulumi.getter(name="domainResolveType")
    def domain_resolve_type(self) -> Optional[pulumi.Input[int]]:
        """
        The domain name resolution method of the access control policy. The policy is enabled by default after it is created. Valid values:
        - **0**: Based on FQDN
        - **1**: DNS-based dynamic resolution
        - **2**: dynamic resolution based on FQDN and DNS.
        """
        return pulumi.get(self, "domain_resolve_type")

    @domain_resolve_type.setter
    def domain_resolve_type(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "domain_resolve_type", value)

    @property
    @pulumi.getter(name="endTime")
    def end_time(self) -> Optional[pulumi.Input[int]]:
        """
        The end time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. Must be full or half time and at least half an hour greater than the start time.
        > **NOTE:**  When RepeatType is set to permit, EndTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, EndTime must have a value and you need to set the end time.
        """
        return pulumi.get(self, "end_time")

    @end_time.setter
    def end_time(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "end_time", value)

    @property
    @pulumi.getter(name="ipVersion")
    def ip_version(self) -> Optional[pulumi.Input[str]]:
        """
        Supported IP address version. Value:
        - **4** (default): indicates the IPv4 address.
        """
        return pulumi.get(self, "ip_version")

    @ip_version.setter
    def ip_version(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ip_version", value)

    @property
    @pulumi.getter
    def release(self) -> Optional[pulumi.Input[str]]:
        """
        The enabled status of the access control policy. The policy is enabled by default after it is created. Value:
        - **true**: Enable access control policy
        - **false**: Do not enable access control policies.
        """
        return pulumi.get(self, "release")

    @release.setter
    def release(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "release", value)

    @property
    @pulumi.getter(name="repeatDays")
    def repeat_days(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[int]]]]:
        """
        Collection of recurring dates for the policy validity period of the access control policy.
        - When RepeatType is 'Permanent', 'None', 'Daily', RepeatDays is an empty collection. For example:[]
        - When RepeatType is Weekly, RepeatDays cannot be empty. For example:["0", "6"]. When the RepeatType is set to Weekly, RepeatDays cannot be repeated.
        - RepeatDays cannot be empty when RepeatType is 'Monthly. For example:[1, 31]. When RepeatType is set to Monthly, RepeatDays cannot be repeated.
        """
        return pulumi.get(self, "repeat_days")

    @repeat_days.setter
    def repeat_days(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[int]]]]):
        pulumi.set(self, "repeat_days", value)

    @property
    @pulumi.getter(name="repeatEndTime")
    def repeat_end_time(self) -> Optional[pulumi.Input[str]]:
        """
        The recurring end time of the policy validity period of the access control policy. For example: 23:30, it must be the whole point or half point time, and at least half an hour greater than the repeat start time.
        > **NOTE:**  When RepeatType is set to normal or None, RepeatEndTime is null. When the RepeatType is Daily, Weekly, or Monthly, the RepeatEndTime must have a value, and you need to set the repeat end time.
        """
        return pulumi.get(self, "repeat_end_time")

    @repeat_end_time.setter
    def repeat_end_time(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "repeat_end_time", value)

    @property
    @pulumi.getter(name="repeatStartTime")
    def repeat_start_time(self) -> Optional[pulumi.Input[str]]:
        """
        The recurring start time of the policy validity period of the access control policy. For example: 08:00, it must be the whole point or half point time, and at least half an hour less than the repeat end time.
        > **NOTE:**  When RepeatType is set to permit or None, RepeatStartTime is empty. When the RepeatType is Daily, Weekly, or Monthly, the RepeatStartTime must have a value and you need to set the repeat start time.
        """
        return pulumi.get(self, "repeat_start_time")

    @repeat_start_time.setter
    def repeat_start_time(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "repeat_start_time", value)

    @property
    @pulumi.getter(name="repeatType")
    def repeat_type(self) -> Optional[pulumi.Input[str]]:
        """
        The type of repetition for the policy validity period of the access control policy. Value:
        - **Permit** (default): Always
        - **None**: Specify a single time
        - **Daily**: Daily
        - **Weekly**: Weekly
        - **Monthly**: Monthly.
        """
        return pulumi.get(self, "repeat_type")

    @repeat_type.setter
    def repeat_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "repeat_type", value)

    @property
    @pulumi.getter(name="startTime")
    def start_time(self) -> Optional[pulumi.Input[int]]:
        """
        The start time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. It must be a full or half hour and at least half an hour less than the end time.
        > **NOTE:**  When RepeatType is set to normal, StartTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, StartTime must have a value and you need to set the start time.
        """
        return pulumi.get(self, "start_time")

    @start_time.setter
    def start_time(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "start_time", value)


@pulumi.input_type
class _NatFirewallControlPolicyState:
    def __init__(__self__, *,
                 acl_action: Optional[pulumi.Input[str]] = None,
                 acl_uuid: Optional[pulumi.Input[str]] = None,
                 application_name_lists: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 create_time: Optional[pulumi.Input[int]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 dest_port: Optional[pulumi.Input[str]] = None,
                 dest_port_group: Optional[pulumi.Input[str]] = None,
                 dest_port_type: Optional[pulumi.Input[str]] = None,
                 destination: Optional[pulumi.Input[str]] = None,
                 destination_type: Optional[pulumi.Input[str]] = None,
                 direction: Optional[pulumi.Input[str]] = None,
                 domain_resolve_type: Optional[pulumi.Input[int]] = None,
                 end_time: Optional[pulumi.Input[int]] = None,
                 ip_version: Optional[pulumi.Input[str]] = None,
                 nat_gateway_id: Optional[pulumi.Input[str]] = None,
                 new_order: Optional[pulumi.Input[str]] = None,
                 proto: Optional[pulumi.Input[str]] = None,
                 release: Optional[pulumi.Input[str]] = None,
                 repeat_days: Optional[pulumi.Input[Sequence[pulumi.Input[int]]]] = None,
                 repeat_end_time: Optional[pulumi.Input[str]] = None,
                 repeat_start_time: Optional[pulumi.Input[str]] = None,
                 repeat_type: Optional[pulumi.Input[str]] = None,
                 source: Optional[pulumi.Input[str]] = None,
                 source_type: Optional[pulumi.Input[str]] = None,
                 start_time: Optional[pulumi.Input[int]] = None):
        """
        Input properties used for looking up and filtering NatFirewallControlPolicy resources.
        :param pulumi.Input[str] acl_action: The method (action) of access traffic passing through Cloud Firewall in the security access control policy. Valid values:
               - **accept**: Release
               - **drop**: Refused
               - **log**: Observation.
        :param pulumi.Input[str] acl_uuid: The unique ID of the security access control policy.
               > **NOTE:**  To modify a security access control policy, you need to provide the unique ID of the policy. You can call the DescribeNatFirewallControlPolicy interface to obtain the ID.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] application_name_lists: The list of application types supported by the access control policy.
        :param pulumi.Input[int] create_time: The time when the policy was created.
        :param pulumi.Input[str] description: The description of the access control policy.
        :param pulumi.Input[str] dest_port: The destination port of traffic access in the access control policy. Value:
               - When the protocol type is set to ICMP, the value of DestPort is null.
               > **NOTE:**  When the protocol type is ICMP, access control on the destination port is not supported.
               - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) IS group, the value of DestPort is null.
               > **NOTE:**  When you select group (destination port address book) for the destination port type of the access control policy, you do not need to set a specific destination port number. All ports that need to be controlled by this access control policy are included in the destination port address book.
               - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) is port, the value of DestPort is the destination port number.
        :param pulumi.Input[str] dest_port_group: The address book name of the destination port of the access traffic in the access control policy.
               > **NOTE:**  When DestPortType is set to group, you need to set the destination port address book name.
        :param pulumi.Input[str] dest_port_type: The destination port type of the access traffic in the security access control policy.
               - **port**: port
               - **group**: Port Address Book.
        :param pulumi.Input[str] destination: The destination address segment in the access control policy. Valid values:
               - When DestinationType is net, Destination is the Destination CIDR. For example: 1.2.XX.XX/24
               - When DestinationType IS group, Destination is the name of the Destination address book. For example: db_group
               - When DestinationType is domain, Destination is the Destination domain name. For example: * .aliyuncs.com
               - When DestinationType is location, Destination is the Destination region. For example: \\["BJ11", "ZB"\\].
        :param pulumi.Input[str] destination_type: The destination address type in the access control policy. Valid values:
               - **net**: Destination Network segment (CIDR address)
               - **group**: Destination Address Book
               - **domain**: the destination domain name.
        :param pulumi.Input[str] direction: The traffic direction of the access control policy. Valid values:
               - **out**: Internal and external traffic access control.
        :param pulumi.Input[int] domain_resolve_type: The domain name resolution method of the access control policy. The policy is enabled by default after it is created. Valid values:
               - **0**: Based on FQDN
               - **1**: DNS-based dynamic resolution
               - **2**: dynamic resolution based on FQDN and DNS.
        :param pulumi.Input[int] end_time: The end time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. Must be full or half time and at least half an hour greater than the start time.
               > **NOTE:**  When RepeatType is set to permit, EndTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, EndTime must have a value and you need to set the end time.
        :param pulumi.Input[str] ip_version: Supported IP address version. Value:
               - **4** (default): indicates the IPv4 address.
        :param pulumi.Input[str] nat_gateway_id: The ID of the NAT gateway instance.
        :param pulumi.Input[str] new_order: The priority for the access control policy to take effect. The priority number increases sequentially from 1, and the smaller the priority number, the higher the priority.
        :param pulumi.Input[str] proto: The security protocol type for traffic access in the access control policy. Valid values:
               - ANY (indicates that all protocol types are queried)
               - TCP
               - UDP
               - ICMP.
        :param pulumi.Input[str] release: The enabled status of the access control policy. The policy is enabled by default after it is created. Value:
               - **true**: Enable access control policy
               - **false**: Do not enable access control policies.
        :param pulumi.Input[Sequence[pulumi.Input[int]]] repeat_days: Collection of recurring dates for the policy validity period of the access control policy.
               - When RepeatType is 'Permanent', 'None', 'Daily', RepeatDays is an empty collection. For example:[]
               - When RepeatType is Weekly, RepeatDays cannot be empty. For example:["0", "6"]. When the RepeatType is set to Weekly, RepeatDays cannot be repeated.
               - RepeatDays cannot be empty when RepeatType is 'Monthly. For example:[1, 31]. When RepeatType is set to Monthly, RepeatDays cannot be repeated.
        :param pulumi.Input[str] repeat_end_time: The recurring end time of the policy validity period of the access control policy. For example: 23:30, it must be the whole point or half point time, and at least half an hour greater than the repeat start time.
               > **NOTE:**  When RepeatType is set to normal or None, RepeatEndTime is null. When the RepeatType is Daily, Weekly, or Monthly, the RepeatEndTime must have a value, and you need to set the repeat end time.
        :param pulumi.Input[str] repeat_start_time: The recurring start time of the policy validity period of the access control policy. For example: 08:00, it must be the whole point or half point time, and at least half an hour less than the repeat end time.
               > **NOTE:**  When RepeatType is set to permit or None, RepeatStartTime is empty. When the RepeatType is Daily, Weekly, or Monthly, the RepeatStartTime must have a value and you need to set the repeat start time.
        :param pulumi.Input[str] repeat_type: The type of repetition for the policy validity period of the access control policy. Value:
               - **Permit** (default): Always
               - **None**: Specify a single time
               - **Daily**: Daily
               - **Weekly**: Weekly
               - **Monthly**: Monthly.
        :param pulumi.Input[str] source: The source address in the access control policy. Valid values:
               - When **SourceType** is set to 'net', Source is the Source CIDR address. For example: 10.2.4.0/24
               - When **SourceType** is set to 'group', Source is the name of the Source address book. For example: db_group.
        :param pulumi.Input[str] source_type: The source address type in the access control policy. Valid values:
               - **net**: the source network segment (CIDR address)
               - **group**: source address book
        :param pulumi.Input[int] start_time: The start time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. It must be a full or half hour and at least half an hour less than the end time.
               > **NOTE:**  When RepeatType is set to normal, StartTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, StartTime must have a value and you need to set the start time.
        """
        if acl_action is not None:
            pulumi.set(__self__, "acl_action", acl_action)
        if acl_uuid is not None:
            pulumi.set(__self__, "acl_uuid", acl_uuid)
        if application_name_lists is not None:
            pulumi.set(__self__, "application_name_lists", application_name_lists)
        if create_time is not None:
            pulumi.set(__self__, "create_time", create_time)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if dest_port is not None:
            pulumi.set(__self__, "dest_port", dest_port)
        if dest_port_group is not None:
            pulumi.set(__self__, "dest_port_group", dest_port_group)
        if dest_port_type is not None:
            pulumi.set(__self__, "dest_port_type", dest_port_type)
        if destination is not None:
            pulumi.set(__self__, "destination", destination)
        if destination_type is not None:
            pulumi.set(__self__, "destination_type", destination_type)
        if direction is not None:
            pulumi.set(__self__, "direction", direction)
        if domain_resolve_type is not None:
            pulumi.set(__self__, "domain_resolve_type", domain_resolve_type)
        if end_time is not None:
            pulumi.set(__self__, "end_time", end_time)
        if ip_version is not None:
            pulumi.set(__self__, "ip_version", ip_version)
        if nat_gateway_id is not None:
            pulumi.set(__self__, "nat_gateway_id", nat_gateway_id)
        if new_order is not None:
            pulumi.set(__self__, "new_order", new_order)
        if proto is not None:
            pulumi.set(__self__, "proto", proto)
        if release is not None:
            pulumi.set(__self__, "release", release)
        if repeat_days is not None:
            pulumi.set(__self__, "repeat_days", repeat_days)
        if repeat_end_time is not None:
            pulumi.set(__self__, "repeat_end_time", repeat_end_time)
        if repeat_start_time is not None:
            pulumi.set(__self__, "repeat_start_time", repeat_start_time)
        if repeat_type is not None:
            pulumi.set(__self__, "repeat_type", repeat_type)
        if source is not None:
            pulumi.set(__self__, "source", source)
        if source_type is not None:
            pulumi.set(__self__, "source_type", source_type)
        if start_time is not None:
            pulumi.set(__self__, "start_time", start_time)

    @property
    @pulumi.getter(name="aclAction")
    def acl_action(self) -> Optional[pulumi.Input[str]]:
        """
        The method (action) of access traffic passing through Cloud Firewall in the security access control policy. Valid values:
        - **accept**: Release
        - **drop**: Refused
        - **log**: Observation.
        """
        return pulumi.get(self, "acl_action")

    @acl_action.setter
    def acl_action(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "acl_action", value)

    @property
    @pulumi.getter(name="aclUuid")
    def acl_uuid(self) -> Optional[pulumi.Input[str]]:
        """
        The unique ID of the security access control policy.
        > **NOTE:**  To modify a security access control policy, you need to provide the unique ID of the policy. You can call the DescribeNatFirewallControlPolicy interface to obtain the ID.
        """
        return pulumi.get(self, "acl_uuid")

    @acl_uuid.setter
    def acl_uuid(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "acl_uuid", value)

    @property
    @pulumi.getter(name="applicationNameLists")
    def application_name_lists(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The list of application types supported by the access control policy.
        """
        return pulumi.get(self, "application_name_lists")

    @application_name_lists.setter
    def application_name_lists(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "application_name_lists", value)

    @property
    @pulumi.getter(name="createTime")
    def create_time(self) -> Optional[pulumi.Input[int]]:
        """
        The time when the policy was created.
        """
        return pulumi.get(self, "create_time")

    @create_time.setter
    def create_time(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "create_time", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the access control policy.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="destPort")
    def dest_port(self) -> Optional[pulumi.Input[str]]:
        """
        The destination port of traffic access in the access control policy. Value:
        - When the protocol type is set to ICMP, the value of DestPort is null.
        > **NOTE:**  When the protocol type is ICMP, access control on the destination port is not supported.
        - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) IS group, the value of DestPort is null.
        > **NOTE:**  When you select group (destination port address book) for the destination port type of the access control policy, you do not need to set a specific destination port number. All ports that need to be controlled by this access control policy are included in the destination port address book.
        - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) is port, the value of DestPort is the destination port number.
        """
        return pulumi.get(self, "dest_port")

    @dest_port.setter
    def dest_port(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "dest_port", value)

    @property
    @pulumi.getter(name="destPortGroup")
    def dest_port_group(self) -> Optional[pulumi.Input[str]]:
        """
        The address book name of the destination port of the access traffic in the access control policy.
        > **NOTE:**  When DestPortType is set to group, you need to set the destination port address book name.
        """
        return pulumi.get(self, "dest_port_group")

    @dest_port_group.setter
    def dest_port_group(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "dest_port_group", value)

    @property
    @pulumi.getter(name="destPortType")
    def dest_port_type(self) -> Optional[pulumi.Input[str]]:
        """
        The destination port type of the access traffic in the security access control policy.
        - **port**: port
        - **group**: Port Address Book.
        """
        return pulumi.get(self, "dest_port_type")

    @dest_port_type.setter
    def dest_port_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "dest_port_type", value)

    @property
    @pulumi.getter
    def destination(self) -> Optional[pulumi.Input[str]]:
        """
        The destination address segment in the access control policy. Valid values:
        - When DestinationType is net, Destination is the Destination CIDR. For example: 1.2.XX.XX/24
        - When DestinationType IS group, Destination is the name of the Destination address book. For example: db_group
        - When DestinationType is domain, Destination is the Destination domain name. For example: * .aliyuncs.com
        - When DestinationType is location, Destination is the Destination region. For example: \\["BJ11", "ZB"\\].
        """
        return pulumi.get(self, "destination")

    @destination.setter
    def destination(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "destination", value)

    @property
    @pulumi.getter(name="destinationType")
    def destination_type(self) -> Optional[pulumi.Input[str]]:
        """
        The destination address type in the access control policy. Valid values:
        - **net**: Destination Network segment (CIDR address)
        - **group**: Destination Address Book
        - **domain**: the destination domain name.
        """
        return pulumi.get(self, "destination_type")

    @destination_type.setter
    def destination_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "destination_type", value)

    @property
    @pulumi.getter
    def direction(self) -> Optional[pulumi.Input[str]]:
        """
        The traffic direction of the access control policy. Valid values:
        - **out**: Internal and external traffic access control.
        """
        return pulumi.get(self, "direction")

    @direction.setter
    def direction(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "direction", value)

    @property
    @pulumi.getter(name="domainResolveType")
    def domain_resolve_type(self) -> Optional[pulumi.Input[int]]:
        """
        The domain name resolution method of the access control policy. The policy is enabled by default after it is created. Valid values:
        - **0**: Based on FQDN
        - **1**: DNS-based dynamic resolution
        - **2**: dynamic resolution based on FQDN and DNS.
        """
        return pulumi.get(self, "domain_resolve_type")

    @domain_resolve_type.setter
    def domain_resolve_type(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "domain_resolve_type", value)

    @property
    @pulumi.getter(name="endTime")
    def end_time(self) -> Optional[pulumi.Input[int]]:
        """
        The end time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. Must be full or half time and at least half an hour greater than the start time.
        > **NOTE:**  When RepeatType is set to permit, EndTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, EndTime must have a value and you need to set the end time.
        """
        return pulumi.get(self, "end_time")

    @end_time.setter
    def end_time(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "end_time", value)

    @property
    @pulumi.getter(name="ipVersion")
    def ip_version(self) -> Optional[pulumi.Input[str]]:
        """
        Supported IP address version. Value:
        - **4** (default): indicates the IPv4 address.
        """
        return pulumi.get(self, "ip_version")

    @ip_version.setter
    def ip_version(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ip_version", value)

    @property
    @pulumi.getter(name="natGatewayId")
    def nat_gateway_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the NAT gateway instance.
        """
        return pulumi.get(self, "nat_gateway_id")

    @nat_gateway_id.setter
    def nat_gateway_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "nat_gateway_id", value)

    @property
    @pulumi.getter(name="newOrder")
    def new_order(self) -> Optional[pulumi.Input[str]]:
        """
        The priority for the access control policy to take effect. The priority number increases sequentially from 1, and the smaller the priority number, the higher the priority.
        """
        return pulumi.get(self, "new_order")

    @new_order.setter
    def new_order(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "new_order", value)

    @property
    @pulumi.getter
    def proto(self) -> Optional[pulumi.Input[str]]:
        """
        The security protocol type for traffic access in the access control policy. Valid values:
        - ANY (indicates that all protocol types are queried)
        - TCP
        - UDP
        - ICMP.
        """
        return pulumi.get(self, "proto")

    @proto.setter
    def proto(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "proto", value)

    @property
    @pulumi.getter
    def release(self) -> Optional[pulumi.Input[str]]:
        """
        The enabled status of the access control policy. The policy is enabled by default after it is created. Value:
        - **true**: Enable access control policy
        - **false**: Do not enable access control policies.
        """
        return pulumi.get(self, "release")

    @release.setter
    def release(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "release", value)

    @property
    @pulumi.getter(name="repeatDays")
    def repeat_days(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[int]]]]:
        """
        Collection of recurring dates for the policy validity period of the access control policy.
        - When RepeatType is 'Permanent', 'None', 'Daily', RepeatDays is an empty collection. For example:[]
        - When RepeatType is Weekly, RepeatDays cannot be empty. For example:["0", "6"]. When the RepeatType is set to Weekly, RepeatDays cannot be repeated.
        - RepeatDays cannot be empty when RepeatType is 'Monthly. For example:[1, 31]. When RepeatType is set to Monthly, RepeatDays cannot be repeated.
        """
        return pulumi.get(self, "repeat_days")

    @repeat_days.setter
    def repeat_days(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[int]]]]):
        pulumi.set(self, "repeat_days", value)

    @property
    @pulumi.getter(name="repeatEndTime")
    def repeat_end_time(self) -> Optional[pulumi.Input[str]]:
        """
        The recurring end time of the policy validity period of the access control policy. For example: 23:30, it must be the whole point or half point time, and at least half an hour greater than the repeat start time.
        > **NOTE:**  When RepeatType is set to normal or None, RepeatEndTime is null. When the RepeatType is Daily, Weekly, or Monthly, the RepeatEndTime must have a value, and you need to set the repeat end time.
        """
        return pulumi.get(self, "repeat_end_time")

    @repeat_end_time.setter
    def repeat_end_time(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "repeat_end_time", value)

    @property
    @pulumi.getter(name="repeatStartTime")
    def repeat_start_time(self) -> Optional[pulumi.Input[str]]:
        """
        The recurring start time of the policy validity period of the access control policy. For example: 08:00, it must be the whole point or half point time, and at least half an hour less than the repeat end time.
        > **NOTE:**  When RepeatType is set to permit or None, RepeatStartTime is empty. When the RepeatType is Daily, Weekly, or Monthly, the RepeatStartTime must have a value and you need to set the repeat start time.
        """
        return pulumi.get(self, "repeat_start_time")

    @repeat_start_time.setter
    def repeat_start_time(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "repeat_start_time", value)

    @property
    @pulumi.getter(name="repeatType")
    def repeat_type(self) -> Optional[pulumi.Input[str]]:
        """
        The type of repetition for the policy validity period of the access control policy. Value:
        - **Permit** (default): Always
        - **None**: Specify a single time
        - **Daily**: Daily
        - **Weekly**: Weekly
        - **Monthly**: Monthly.
        """
        return pulumi.get(self, "repeat_type")

    @repeat_type.setter
    def repeat_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "repeat_type", value)

    @property
    @pulumi.getter
    def source(self) -> Optional[pulumi.Input[str]]:
        """
        The source address in the access control policy. Valid values:
        - When **SourceType** is set to 'net', Source is the Source CIDR address. For example: 10.2.4.0/24
        - When **SourceType** is set to 'group', Source is the name of the Source address book. For example: db_group.
        """
        return pulumi.get(self, "source")

    @source.setter
    def source(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "source", value)

    @property
    @pulumi.getter(name="sourceType")
    def source_type(self) -> Optional[pulumi.Input[str]]:
        """
        The source address type in the access control policy. Valid values:
        - **net**: the source network segment (CIDR address)
        - **group**: source address book
        """
        return pulumi.get(self, "source_type")

    @source_type.setter
    def source_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "source_type", value)

    @property
    @pulumi.getter(name="startTime")
    def start_time(self) -> Optional[pulumi.Input[int]]:
        """
        The start time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. It must be a full or half hour and at least half an hour less than the end time.
        > **NOTE:**  When RepeatType is set to normal, StartTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, StartTime must have a value and you need to set the start time.
        """
        return pulumi.get(self, "start_time")

    @start_time.setter
    def start_time(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "start_time", value)


class NatFirewallControlPolicy(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 acl_action: Optional[pulumi.Input[str]] = None,
                 application_name_lists: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 dest_port: Optional[pulumi.Input[str]] = None,
                 dest_port_group: Optional[pulumi.Input[str]] = None,
                 dest_port_type: Optional[pulumi.Input[str]] = None,
                 destination: Optional[pulumi.Input[str]] = None,
                 destination_type: Optional[pulumi.Input[str]] = None,
                 direction: Optional[pulumi.Input[str]] = None,
                 domain_resolve_type: Optional[pulumi.Input[int]] = None,
                 end_time: Optional[pulumi.Input[int]] = None,
                 ip_version: Optional[pulumi.Input[str]] = None,
                 nat_gateway_id: Optional[pulumi.Input[str]] = None,
                 new_order: Optional[pulumi.Input[str]] = None,
                 proto: Optional[pulumi.Input[str]] = None,
                 release: Optional[pulumi.Input[str]] = None,
                 repeat_days: Optional[pulumi.Input[Sequence[pulumi.Input[int]]]] = None,
                 repeat_end_time: Optional[pulumi.Input[str]] = None,
                 repeat_start_time: Optional[pulumi.Input[str]] = None,
                 repeat_type: Optional[pulumi.Input[str]] = None,
                 source: Optional[pulumi.Input[str]] = None,
                 source_type: Optional[pulumi.Input[str]] = None,
                 start_time: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        """
        Provides a Cloud Firewall Nat Firewall Control Policy resource. Nat firewall access control policy.

        For information about Cloud Firewall Nat Firewall Control Policy and how to use it, see [What is Nat Firewall Control Policy](https://www.alibabacloud.com/help/en/cloud-firewall/developer-reference/api-cloudfw-2017-12-07-createnatfirewallcontrolpolicy).

        > **NOTE:** Available since v1.224.0.

        ## Import

        Cloud Firewall Nat Firewall Control Policy can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:cloudfirewall/natFirewallControlPolicy:NatFirewallControlPolicy example <acl_uuid>:<nat_gateway_id>:<direction>
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] acl_action: The method (action) of access traffic passing through Cloud Firewall in the security access control policy. Valid values:
               - **accept**: Release
               - **drop**: Refused
               - **log**: Observation.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] application_name_lists: The list of application types supported by the access control policy.
        :param pulumi.Input[str] description: The description of the access control policy.
        :param pulumi.Input[str] dest_port: The destination port of traffic access in the access control policy. Value:
               - When the protocol type is set to ICMP, the value of DestPort is null.
               > **NOTE:**  When the protocol type is ICMP, access control on the destination port is not supported.
               - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) IS group, the value of DestPort is null.
               > **NOTE:**  When you select group (destination port address book) for the destination port type of the access control policy, you do not need to set a specific destination port number. All ports that need to be controlled by this access control policy are included in the destination port address book.
               - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) is port, the value of DestPort is the destination port number.
        :param pulumi.Input[str] dest_port_group: The address book name of the destination port of the access traffic in the access control policy.
               > **NOTE:**  When DestPortType is set to group, you need to set the destination port address book name.
        :param pulumi.Input[str] dest_port_type: The destination port type of the access traffic in the security access control policy.
               - **port**: port
               - **group**: Port Address Book.
        :param pulumi.Input[str] destination: The destination address segment in the access control policy. Valid values:
               - When DestinationType is net, Destination is the Destination CIDR. For example: 1.2.XX.XX/24
               - When DestinationType IS group, Destination is the name of the Destination address book. For example: db_group
               - When DestinationType is domain, Destination is the Destination domain name. For example: * .aliyuncs.com
               - When DestinationType is location, Destination is the Destination region. For example: \\["BJ11", "ZB"\\].
        :param pulumi.Input[str] destination_type: The destination address type in the access control policy. Valid values:
               - **net**: Destination Network segment (CIDR address)
               - **group**: Destination Address Book
               - **domain**: the destination domain name.
        :param pulumi.Input[str] direction: The traffic direction of the access control policy. Valid values:
               - **out**: Internal and external traffic access control.
        :param pulumi.Input[int] domain_resolve_type: The domain name resolution method of the access control policy. The policy is enabled by default after it is created. Valid values:
               - **0**: Based on FQDN
               - **1**: DNS-based dynamic resolution
               - **2**: dynamic resolution based on FQDN and DNS.
        :param pulumi.Input[int] end_time: The end time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. Must be full or half time and at least half an hour greater than the start time.
               > **NOTE:**  When RepeatType is set to permit, EndTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, EndTime must have a value and you need to set the end time.
        :param pulumi.Input[str] ip_version: Supported IP address version. Value:
               - **4** (default): indicates the IPv4 address.
        :param pulumi.Input[str] nat_gateway_id: The ID of the NAT gateway instance.
        :param pulumi.Input[str] new_order: The priority for the access control policy to take effect. The priority number increases sequentially from 1, and the smaller the priority number, the higher the priority.
        :param pulumi.Input[str] proto: The security protocol type for traffic access in the access control policy. Valid values:
               - ANY (indicates that all protocol types are queried)
               - TCP
               - UDP
               - ICMP.
        :param pulumi.Input[str] release: The enabled status of the access control policy. The policy is enabled by default after it is created. Value:
               - **true**: Enable access control policy
               - **false**: Do not enable access control policies.
        :param pulumi.Input[Sequence[pulumi.Input[int]]] repeat_days: Collection of recurring dates for the policy validity period of the access control policy.
               - When RepeatType is 'Permanent', 'None', 'Daily', RepeatDays is an empty collection. For example:[]
               - When RepeatType is Weekly, RepeatDays cannot be empty. For example:["0", "6"]. When the RepeatType is set to Weekly, RepeatDays cannot be repeated.
               - RepeatDays cannot be empty when RepeatType is 'Monthly. For example:[1, 31]. When RepeatType is set to Monthly, RepeatDays cannot be repeated.
        :param pulumi.Input[str] repeat_end_time: The recurring end time of the policy validity period of the access control policy. For example: 23:30, it must be the whole point or half point time, and at least half an hour greater than the repeat start time.
               > **NOTE:**  When RepeatType is set to normal or None, RepeatEndTime is null. When the RepeatType is Daily, Weekly, or Monthly, the RepeatEndTime must have a value, and you need to set the repeat end time.
        :param pulumi.Input[str] repeat_start_time: The recurring start time of the policy validity period of the access control policy. For example: 08:00, it must be the whole point or half point time, and at least half an hour less than the repeat end time.
               > **NOTE:**  When RepeatType is set to permit or None, RepeatStartTime is empty. When the RepeatType is Daily, Weekly, or Monthly, the RepeatStartTime must have a value and you need to set the repeat start time.
        :param pulumi.Input[str] repeat_type: The type of repetition for the policy validity period of the access control policy. Value:
               - **Permit** (default): Always
               - **None**: Specify a single time
               - **Daily**: Daily
               - **Weekly**: Weekly
               - **Monthly**: Monthly.
        :param pulumi.Input[str] source: The source address in the access control policy. Valid values:
               - When **SourceType** is set to 'net', Source is the Source CIDR address. For example: 10.2.4.0/24
               - When **SourceType** is set to 'group', Source is the name of the Source address book. For example: db_group.
        :param pulumi.Input[str] source_type: The source address type in the access control policy. Valid values:
               - **net**: the source network segment (CIDR address)
               - **group**: source address book
        :param pulumi.Input[int] start_time: The start time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. It must be a full or half hour and at least half an hour less than the end time.
               > **NOTE:**  When RepeatType is set to normal, StartTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, StartTime must have a value and you need to set the start time.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: NatFirewallControlPolicyArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a Cloud Firewall Nat Firewall Control Policy resource. Nat firewall access control policy.

        For information about Cloud Firewall Nat Firewall Control Policy and how to use it, see [What is Nat Firewall Control Policy](https://www.alibabacloud.com/help/en/cloud-firewall/developer-reference/api-cloudfw-2017-12-07-createnatfirewallcontrolpolicy).

        > **NOTE:** Available since v1.224.0.

        ## Import

        Cloud Firewall Nat Firewall Control Policy can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:cloudfirewall/natFirewallControlPolicy:NatFirewallControlPolicy example <acl_uuid>:<nat_gateway_id>:<direction>
        ```

        :param str resource_name: The name of the resource.
        :param NatFirewallControlPolicyArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(NatFirewallControlPolicyArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 acl_action: Optional[pulumi.Input[str]] = None,
                 application_name_lists: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 dest_port: Optional[pulumi.Input[str]] = None,
                 dest_port_group: Optional[pulumi.Input[str]] = None,
                 dest_port_type: Optional[pulumi.Input[str]] = None,
                 destination: Optional[pulumi.Input[str]] = None,
                 destination_type: Optional[pulumi.Input[str]] = None,
                 direction: Optional[pulumi.Input[str]] = None,
                 domain_resolve_type: Optional[pulumi.Input[int]] = None,
                 end_time: Optional[pulumi.Input[int]] = None,
                 ip_version: Optional[pulumi.Input[str]] = None,
                 nat_gateway_id: Optional[pulumi.Input[str]] = None,
                 new_order: Optional[pulumi.Input[str]] = None,
                 proto: Optional[pulumi.Input[str]] = None,
                 release: Optional[pulumi.Input[str]] = None,
                 repeat_days: Optional[pulumi.Input[Sequence[pulumi.Input[int]]]] = None,
                 repeat_end_time: Optional[pulumi.Input[str]] = None,
                 repeat_start_time: Optional[pulumi.Input[str]] = None,
                 repeat_type: Optional[pulumi.Input[str]] = None,
                 source: Optional[pulumi.Input[str]] = None,
                 source_type: Optional[pulumi.Input[str]] = None,
                 start_time: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = NatFirewallControlPolicyArgs.__new__(NatFirewallControlPolicyArgs)

            if acl_action is None and not opts.urn:
                raise TypeError("Missing required property 'acl_action'")
            __props__.__dict__["acl_action"] = acl_action
            if application_name_lists is None and not opts.urn:
                raise TypeError("Missing required property 'application_name_lists'")
            __props__.__dict__["application_name_lists"] = application_name_lists
            if description is None and not opts.urn:
                raise TypeError("Missing required property 'description'")
            __props__.__dict__["description"] = description
            __props__.__dict__["dest_port"] = dest_port
            __props__.__dict__["dest_port_group"] = dest_port_group
            __props__.__dict__["dest_port_type"] = dest_port_type
            if destination is None and not opts.urn:
                raise TypeError("Missing required property 'destination'")
            __props__.__dict__["destination"] = destination
            if destination_type is None and not opts.urn:
                raise TypeError("Missing required property 'destination_type'")
            __props__.__dict__["destination_type"] = destination_type
            if direction is None and not opts.urn:
                raise TypeError("Missing required property 'direction'")
            __props__.__dict__["direction"] = direction
            __props__.__dict__["domain_resolve_type"] = domain_resolve_type
            __props__.__dict__["end_time"] = end_time
            __props__.__dict__["ip_version"] = ip_version
            if nat_gateway_id is None and not opts.urn:
                raise TypeError("Missing required property 'nat_gateway_id'")
            __props__.__dict__["nat_gateway_id"] = nat_gateway_id
            if new_order is None and not opts.urn:
                raise TypeError("Missing required property 'new_order'")
            __props__.__dict__["new_order"] = new_order
            if proto is None and not opts.urn:
                raise TypeError("Missing required property 'proto'")
            __props__.__dict__["proto"] = proto
            __props__.__dict__["release"] = release
            __props__.__dict__["repeat_days"] = repeat_days
            __props__.__dict__["repeat_end_time"] = repeat_end_time
            __props__.__dict__["repeat_start_time"] = repeat_start_time
            __props__.__dict__["repeat_type"] = repeat_type
            if source is None and not opts.urn:
                raise TypeError("Missing required property 'source'")
            __props__.__dict__["source"] = source
            if source_type is None and not opts.urn:
                raise TypeError("Missing required property 'source_type'")
            __props__.__dict__["source_type"] = source_type
            __props__.__dict__["start_time"] = start_time
            __props__.__dict__["acl_uuid"] = None
            __props__.__dict__["create_time"] = None
        super(NatFirewallControlPolicy, __self__).__init__(
            'alicloud:cloudfirewall/natFirewallControlPolicy:NatFirewallControlPolicy',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            acl_action: Optional[pulumi.Input[str]] = None,
            acl_uuid: Optional[pulumi.Input[str]] = None,
            application_name_lists: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            create_time: Optional[pulumi.Input[int]] = None,
            description: Optional[pulumi.Input[str]] = None,
            dest_port: Optional[pulumi.Input[str]] = None,
            dest_port_group: Optional[pulumi.Input[str]] = None,
            dest_port_type: Optional[pulumi.Input[str]] = None,
            destination: Optional[pulumi.Input[str]] = None,
            destination_type: Optional[pulumi.Input[str]] = None,
            direction: Optional[pulumi.Input[str]] = None,
            domain_resolve_type: Optional[pulumi.Input[int]] = None,
            end_time: Optional[pulumi.Input[int]] = None,
            ip_version: Optional[pulumi.Input[str]] = None,
            nat_gateway_id: Optional[pulumi.Input[str]] = None,
            new_order: Optional[pulumi.Input[str]] = None,
            proto: Optional[pulumi.Input[str]] = None,
            release: Optional[pulumi.Input[str]] = None,
            repeat_days: Optional[pulumi.Input[Sequence[pulumi.Input[int]]]] = None,
            repeat_end_time: Optional[pulumi.Input[str]] = None,
            repeat_start_time: Optional[pulumi.Input[str]] = None,
            repeat_type: Optional[pulumi.Input[str]] = None,
            source: Optional[pulumi.Input[str]] = None,
            source_type: Optional[pulumi.Input[str]] = None,
            start_time: Optional[pulumi.Input[int]] = None) -> 'NatFirewallControlPolicy':
        """
        Get an existing NatFirewallControlPolicy resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] acl_action: The method (action) of access traffic passing through Cloud Firewall in the security access control policy. Valid values:
               - **accept**: Release
               - **drop**: Refused
               - **log**: Observation.
        :param pulumi.Input[str] acl_uuid: The unique ID of the security access control policy.
               > **NOTE:**  To modify a security access control policy, you need to provide the unique ID of the policy. You can call the DescribeNatFirewallControlPolicy interface to obtain the ID.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] application_name_lists: The list of application types supported by the access control policy.
        :param pulumi.Input[int] create_time: The time when the policy was created.
        :param pulumi.Input[str] description: The description of the access control policy.
        :param pulumi.Input[str] dest_port: The destination port of traffic access in the access control policy. Value:
               - When the protocol type is set to ICMP, the value of DestPort is null.
               > **NOTE:**  When the protocol type is ICMP, access control on the destination port is not supported.
               - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) IS group, the value of DestPort is null.
               > **NOTE:**  When you select group (destination port address book) for the destination port type of the access control policy, you do not need to set a specific destination port number. All ports that need to be controlled by this access control policy are included in the destination port address book.
               - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) is port, the value of DestPort is the destination port number.
        :param pulumi.Input[str] dest_port_group: The address book name of the destination port of the access traffic in the access control policy.
               > **NOTE:**  When DestPortType is set to group, you need to set the destination port address book name.
        :param pulumi.Input[str] dest_port_type: The destination port type of the access traffic in the security access control policy.
               - **port**: port
               - **group**: Port Address Book.
        :param pulumi.Input[str] destination: The destination address segment in the access control policy. Valid values:
               - When DestinationType is net, Destination is the Destination CIDR. For example: 1.2.XX.XX/24
               - When DestinationType IS group, Destination is the name of the Destination address book. For example: db_group
               - When DestinationType is domain, Destination is the Destination domain name. For example: * .aliyuncs.com
               - When DestinationType is location, Destination is the Destination region. For example: \\["BJ11", "ZB"\\].
        :param pulumi.Input[str] destination_type: The destination address type in the access control policy. Valid values:
               - **net**: Destination Network segment (CIDR address)
               - **group**: Destination Address Book
               - **domain**: the destination domain name.
        :param pulumi.Input[str] direction: The traffic direction of the access control policy. Valid values:
               - **out**: Internal and external traffic access control.
        :param pulumi.Input[int] domain_resolve_type: The domain name resolution method of the access control policy. The policy is enabled by default after it is created. Valid values:
               - **0**: Based on FQDN
               - **1**: DNS-based dynamic resolution
               - **2**: dynamic resolution based on FQDN and DNS.
        :param pulumi.Input[int] end_time: The end time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. Must be full or half time and at least half an hour greater than the start time.
               > **NOTE:**  When RepeatType is set to permit, EndTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, EndTime must have a value and you need to set the end time.
        :param pulumi.Input[str] ip_version: Supported IP address version. Value:
               - **4** (default): indicates the IPv4 address.
        :param pulumi.Input[str] nat_gateway_id: The ID of the NAT gateway instance.
        :param pulumi.Input[str] new_order: The priority for the access control policy to take effect. The priority number increases sequentially from 1, and the smaller the priority number, the higher the priority.
        :param pulumi.Input[str] proto: The security protocol type for traffic access in the access control policy. Valid values:
               - ANY (indicates that all protocol types are queried)
               - TCP
               - UDP
               - ICMP.
        :param pulumi.Input[str] release: The enabled status of the access control policy. The policy is enabled by default after it is created. Value:
               - **true**: Enable access control policy
               - **false**: Do not enable access control policies.
        :param pulumi.Input[Sequence[pulumi.Input[int]]] repeat_days: Collection of recurring dates for the policy validity period of the access control policy.
               - When RepeatType is 'Permanent', 'None', 'Daily', RepeatDays is an empty collection. For example:[]
               - When RepeatType is Weekly, RepeatDays cannot be empty. For example:["0", "6"]. When the RepeatType is set to Weekly, RepeatDays cannot be repeated.
               - RepeatDays cannot be empty when RepeatType is 'Monthly. For example:[1, 31]. When RepeatType is set to Monthly, RepeatDays cannot be repeated.
        :param pulumi.Input[str] repeat_end_time: The recurring end time of the policy validity period of the access control policy. For example: 23:30, it must be the whole point or half point time, and at least half an hour greater than the repeat start time.
               > **NOTE:**  When RepeatType is set to normal or None, RepeatEndTime is null. When the RepeatType is Daily, Weekly, or Monthly, the RepeatEndTime must have a value, and you need to set the repeat end time.
        :param pulumi.Input[str] repeat_start_time: The recurring start time of the policy validity period of the access control policy. For example: 08:00, it must be the whole point or half point time, and at least half an hour less than the repeat end time.
               > **NOTE:**  When RepeatType is set to permit or None, RepeatStartTime is empty. When the RepeatType is Daily, Weekly, or Monthly, the RepeatStartTime must have a value and you need to set the repeat start time.
        :param pulumi.Input[str] repeat_type: The type of repetition for the policy validity period of the access control policy. Value:
               - **Permit** (default): Always
               - **None**: Specify a single time
               - **Daily**: Daily
               - **Weekly**: Weekly
               - **Monthly**: Monthly.
        :param pulumi.Input[str] source: The source address in the access control policy. Valid values:
               - When **SourceType** is set to 'net', Source is the Source CIDR address. For example: 10.2.4.0/24
               - When **SourceType** is set to 'group', Source is the name of the Source address book. For example: db_group.
        :param pulumi.Input[str] source_type: The source address type in the access control policy. Valid values:
               - **net**: the source network segment (CIDR address)
               - **group**: source address book
        :param pulumi.Input[int] start_time: The start time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. It must be a full or half hour and at least half an hour less than the end time.
               > **NOTE:**  When RepeatType is set to normal, StartTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, StartTime must have a value and you need to set the start time.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _NatFirewallControlPolicyState.__new__(_NatFirewallControlPolicyState)

        __props__.__dict__["acl_action"] = acl_action
        __props__.__dict__["acl_uuid"] = acl_uuid
        __props__.__dict__["application_name_lists"] = application_name_lists
        __props__.__dict__["create_time"] = create_time
        __props__.__dict__["description"] = description
        __props__.__dict__["dest_port"] = dest_port
        __props__.__dict__["dest_port_group"] = dest_port_group
        __props__.__dict__["dest_port_type"] = dest_port_type
        __props__.__dict__["destination"] = destination
        __props__.__dict__["destination_type"] = destination_type
        __props__.__dict__["direction"] = direction
        __props__.__dict__["domain_resolve_type"] = domain_resolve_type
        __props__.__dict__["end_time"] = end_time
        __props__.__dict__["ip_version"] = ip_version
        __props__.__dict__["nat_gateway_id"] = nat_gateway_id
        __props__.__dict__["new_order"] = new_order
        __props__.__dict__["proto"] = proto
        __props__.__dict__["release"] = release
        __props__.__dict__["repeat_days"] = repeat_days
        __props__.__dict__["repeat_end_time"] = repeat_end_time
        __props__.__dict__["repeat_start_time"] = repeat_start_time
        __props__.__dict__["repeat_type"] = repeat_type
        __props__.__dict__["source"] = source
        __props__.__dict__["source_type"] = source_type
        __props__.__dict__["start_time"] = start_time
        return NatFirewallControlPolicy(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="aclAction")
    def acl_action(self) -> pulumi.Output[str]:
        """
        The method (action) of access traffic passing through Cloud Firewall in the security access control policy. Valid values:
        - **accept**: Release
        - **drop**: Refused
        - **log**: Observation.
        """
        return pulumi.get(self, "acl_action")

    @property
    @pulumi.getter(name="aclUuid")
    def acl_uuid(self) -> pulumi.Output[str]:
        """
        The unique ID of the security access control policy.
        > **NOTE:**  To modify a security access control policy, you need to provide the unique ID of the policy. You can call the DescribeNatFirewallControlPolicy interface to obtain the ID.
        """
        return pulumi.get(self, "acl_uuid")

    @property
    @pulumi.getter(name="applicationNameLists")
    def application_name_lists(self) -> pulumi.Output[Sequence[str]]:
        """
        The list of application types supported by the access control policy.
        """
        return pulumi.get(self, "application_name_lists")

    @property
    @pulumi.getter(name="createTime")
    def create_time(self) -> pulumi.Output[int]:
        """
        The time when the policy was created.
        """
        return pulumi.get(self, "create_time")

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[str]:
        """
        The description of the access control policy.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="destPort")
    def dest_port(self) -> pulumi.Output[str]:
        """
        The destination port of traffic access in the access control policy. Value:
        - When the protocol type is set to ICMP, the value of DestPort is null.
        > **NOTE:**  When the protocol type is ICMP, access control on the destination port is not supported.
        - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) IS group, the value of DestPort is null.
        > **NOTE:**  When you select group (destination port address book) for the destination port type of the access control policy, you do not need to set a specific destination port number. All ports that need to be controlled by this access control policy are included in the destination port address book.
        - When the protocol type is TCP, UDP, or ANY, and the destination port type (DestPortType) is port, the value of DestPort is the destination port number.
        """
        return pulumi.get(self, "dest_port")

    @property
    @pulumi.getter(name="destPortGroup")
    def dest_port_group(self) -> pulumi.Output[Optional[str]]:
        """
        The address book name of the destination port of the access traffic in the access control policy.
        > **NOTE:**  When DestPortType is set to group, you need to set the destination port address book name.
        """
        return pulumi.get(self, "dest_port_group")

    @property
    @pulumi.getter(name="destPortType")
    def dest_port_type(self) -> pulumi.Output[str]:
        """
        The destination port type of the access traffic in the security access control policy.
        - **port**: port
        - **group**: Port Address Book.
        """
        return pulumi.get(self, "dest_port_type")

    @property
    @pulumi.getter
    def destination(self) -> pulumi.Output[str]:
        """
        The destination address segment in the access control policy. Valid values:
        - When DestinationType is net, Destination is the Destination CIDR. For example: 1.2.XX.XX/24
        - When DestinationType IS group, Destination is the name of the Destination address book. For example: db_group
        - When DestinationType is domain, Destination is the Destination domain name. For example: * .aliyuncs.com
        - When DestinationType is location, Destination is the Destination region. For example: \\["BJ11", "ZB"\\].
        """
        return pulumi.get(self, "destination")

    @property
    @pulumi.getter(name="destinationType")
    def destination_type(self) -> pulumi.Output[str]:
        """
        The destination address type in the access control policy. Valid values:
        - **net**: Destination Network segment (CIDR address)
        - **group**: Destination Address Book
        - **domain**: the destination domain name.
        """
        return pulumi.get(self, "destination_type")

    @property
    @pulumi.getter
    def direction(self) -> pulumi.Output[str]:
        """
        The traffic direction of the access control policy. Valid values:
        - **out**: Internal and external traffic access control.
        """
        return pulumi.get(self, "direction")

    @property
    @pulumi.getter(name="domainResolveType")
    def domain_resolve_type(self) -> pulumi.Output[Optional[int]]:
        """
        The domain name resolution method of the access control policy. The policy is enabled by default after it is created. Valid values:
        - **0**: Based on FQDN
        - **1**: DNS-based dynamic resolution
        - **2**: dynamic resolution based on FQDN and DNS.
        """
        return pulumi.get(self, "domain_resolve_type")

    @property
    @pulumi.getter(name="endTime")
    def end_time(self) -> pulumi.Output[Optional[int]]:
        """
        The end time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. Must be full or half time and at least half an hour greater than the start time.
        > **NOTE:**  When RepeatType is set to permit, EndTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, EndTime must have a value and you need to set the end time.
        """
        return pulumi.get(self, "end_time")

    @property
    @pulumi.getter(name="ipVersion")
    def ip_version(self) -> pulumi.Output[Optional[str]]:
        """
        Supported IP address version. Value:
        - **4** (default): indicates the IPv4 address.
        """
        return pulumi.get(self, "ip_version")

    @property
    @pulumi.getter(name="natGatewayId")
    def nat_gateway_id(self) -> pulumi.Output[str]:
        """
        The ID of the NAT gateway instance.
        """
        return pulumi.get(self, "nat_gateway_id")

    @property
    @pulumi.getter(name="newOrder")
    def new_order(self) -> pulumi.Output[str]:
        """
        The priority for the access control policy to take effect. The priority number increases sequentially from 1, and the smaller the priority number, the higher the priority.
        """
        return pulumi.get(self, "new_order")

    @property
    @pulumi.getter
    def proto(self) -> pulumi.Output[str]:
        """
        The security protocol type for traffic access in the access control policy. Valid values:
        - ANY (indicates that all protocol types are queried)
        - TCP
        - UDP
        - ICMP.
        """
        return pulumi.get(self, "proto")

    @property
    @pulumi.getter
    def release(self) -> pulumi.Output[str]:
        """
        The enabled status of the access control policy. The policy is enabled by default after it is created. Value:
        - **true**: Enable access control policy
        - **false**: Do not enable access control policies.
        """
        return pulumi.get(self, "release")

    @property
    @pulumi.getter(name="repeatDays")
    def repeat_days(self) -> pulumi.Output[Optional[Sequence[int]]]:
        """
        Collection of recurring dates for the policy validity period of the access control policy.
        - When RepeatType is 'Permanent', 'None', 'Daily', RepeatDays is an empty collection. For example:[]
        - When RepeatType is Weekly, RepeatDays cannot be empty. For example:["0", "6"]. When the RepeatType is set to Weekly, RepeatDays cannot be repeated.
        - RepeatDays cannot be empty when RepeatType is 'Monthly. For example:[1, 31]. When RepeatType is set to Monthly, RepeatDays cannot be repeated.
        """
        return pulumi.get(self, "repeat_days")

    @property
    @pulumi.getter(name="repeatEndTime")
    def repeat_end_time(self) -> pulumi.Output[Optional[str]]:
        """
        The recurring end time of the policy validity period of the access control policy. For example: 23:30, it must be the whole point or half point time, and at least half an hour greater than the repeat start time.
        > **NOTE:**  When RepeatType is set to normal or None, RepeatEndTime is null. When the RepeatType is Daily, Weekly, or Monthly, the RepeatEndTime must have a value, and you need to set the repeat end time.
        """
        return pulumi.get(self, "repeat_end_time")

    @property
    @pulumi.getter(name="repeatStartTime")
    def repeat_start_time(self) -> pulumi.Output[Optional[str]]:
        """
        The recurring start time of the policy validity period of the access control policy. For example: 08:00, it must be the whole point or half point time, and at least half an hour less than the repeat end time.
        > **NOTE:**  When RepeatType is set to permit or None, RepeatStartTime is empty. When the RepeatType is Daily, Weekly, or Monthly, the RepeatStartTime must have a value and you need to set the repeat start time.
        """
        return pulumi.get(self, "repeat_start_time")

    @property
    @pulumi.getter(name="repeatType")
    def repeat_type(self) -> pulumi.Output[str]:
        """
        The type of repetition for the policy validity period of the access control policy. Value:
        - **Permit** (default): Always
        - **None**: Specify a single time
        - **Daily**: Daily
        - **Weekly**: Weekly
        - **Monthly**: Monthly.
        """
        return pulumi.get(self, "repeat_type")

    @property
    @pulumi.getter
    def source(self) -> pulumi.Output[str]:
        """
        The source address in the access control policy. Valid values:
        - When **SourceType** is set to 'net', Source is the Source CIDR address. For example: 10.2.4.0/24
        - When **SourceType** is set to 'group', Source is the name of the Source address book. For example: db_group.
        """
        return pulumi.get(self, "source")

    @property
    @pulumi.getter(name="sourceType")
    def source_type(self) -> pulumi.Output[str]:
        """
        The source address type in the access control policy. Valid values:
        - **net**: the source network segment (CIDR address)
        - **group**: source address book
        """
        return pulumi.get(self, "source_type")

    @property
    @pulumi.getter(name="startTime")
    def start_time(self) -> pulumi.Output[Optional[int]]:
        """
        The start time of the policy validity period of the access control policy. Expresses using the second-level timestamp format. It must be a full or half hour and at least half an hour less than the end time.
        > **NOTE:**  When RepeatType is set to normal, StartTime is null. When the RepeatType is None, Daily, Weekly, or Monthly, StartTime must have a value and you need to set the start time.
        """
        return pulumi.get(self, "start_time")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['ApplicationScalingRuleArgs', 'ApplicationScalingRule']

@pulumi.input_type
class ApplicationScalingRuleArgs:
    def __init__(__self__, *,
                 app_id: pulumi.Input[str],
                 scaling_rule_name: pulumi.Input[str],
                 scaling_rule_type: pulumi.Input[str],
                 min_ready_instance_ratio: Optional[pulumi.Input[int]] = None,
                 min_ready_instances: Optional[pulumi.Input[int]] = None,
                 scaling_rule_enable: Optional[pulumi.Input[bool]] = None,
                 scaling_rule_metric: Optional[pulumi.Input['ApplicationScalingRuleScalingRuleMetricArgs']] = None,
                 scaling_rule_timer: Optional[pulumi.Input['ApplicationScalingRuleScalingRuleTimerArgs']] = None):
        """
        The set of arguments for constructing a ApplicationScalingRule resource.
        :param pulumi.Input[str] app_id: Application ID.
        :param pulumi.Input[str] scaling_rule_name: The name of a custom elastic scaling policy. In the application, the policy name cannot be repeated. It must start with a lowercase letter, and can only contain lowercase letters, numbers, and dashes (-), and no more than 32 characters. After the scaling policy is successfully created, the policy name cannot be modified.
        :param pulumi.Input[str] scaling_rule_type: Flexible strategy type. Valid values: `mix`, `timing` and `metric`.
        :param pulumi.Input[int] min_ready_instance_ratio: The min ready instance ratio.
        :param pulumi.Input[int] min_ready_instances: The min ready instances.
        :param pulumi.Input[bool] scaling_rule_enable: True whether the auto scaling policy is enabled. The value description is as follows: true: enabled state. false: disabled status. Valid values: `false`, `true`.
        :param pulumi.Input['ApplicationScalingRuleScalingRuleMetricArgs'] scaling_rule_metric: Monitor the configuration of the indicator elasticity strategy. See `scaling_rule_metric` below.
        :param pulumi.Input['ApplicationScalingRuleScalingRuleTimerArgs'] scaling_rule_timer: Configuration of Timing Resilient Policies. See `scaling_rule_timer` below.
        """
        pulumi.set(__self__, "app_id", app_id)
        pulumi.set(__self__, "scaling_rule_name", scaling_rule_name)
        pulumi.set(__self__, "scaling_rule_type", scaling_rule_type)
        if min_ready_instance_ratio is not None:
            pulumi.set(__self__, "min_ready_instance_ratio", min_ready_instance_ratio)
        if min_ready_instances is not None:
            pulumi.set(__self__, "min_ready_instances", min_ready_instances)
        if scaling_rule_enable is not None:
            pulumi.set(__self__, "scaling_rule_enable", scaling_rule_enable)
        if scaling_rule_metric is not None:
            pulumi.set(__self__, "scaling_rule_metric", scaling_rule_metric)
        if scaling_rule_timer is not None:
            pulumi.set(__self__, "scaling_rule_timer", scaling_rule_timer)

    @property
    @pulumi.getter(name="appId")
    def app_id(self) -> pulumi.Input[str]:
        """
        Application ID.
        """
        return pulumi.get(self, "app_id")

    @app_id.setter
    def app_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "app_id", value)

    @property
    @pulumi.getter(name="scalingRuleName")
    def scaling_rule_name(self) -> pulumi.Input[str]:
        """
        The name of a custom elastic scaling policy. In the application, the policy name cannot be repeated. It must start with a lowercase letter, and can only contain lowercase letters, numbers, and dashes (-), and no more than 32 characters. After the scaling policy is successfully created, the policy name cannot be modified.
        """
        return pulumi.get(self, "scaling_rule_name")

    @scaling_rule_name.setter
    def scaling_rule_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "scaling_rule_name", value)

    @property
    @pulumi.getter(name="scalingRuleType")
    def scaling_rule_type(self) -> pulumi.Input[str]:
        """
        Flexible strategy type. Valid values: `mix`, `timing` and `metric`.
        """
        return pulumi.get(self, "scaling_rule_type")

    @scaling_rule_type.setter
    def scaling_rule_type(self, value: pulumi.Input[str]):
        pulumi.set(self, "scaling_rule_type", value)

    @property
    @pulumi.getter(name="minReadyInstanceRatio")
    def min_ready_instance_ratio(self) -> Optional[pulumi.Input[int]]:
        """
        The min ready instance ratio.
        """
        return pulumi.get(self, "min_ready_instance_ratio")

    @min_ready_instance_ratio.setter
    def min_ready_instance_ratio(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "min_ready_instance_ratio", value)

    @property
    @pulumi.getter(name="minReadyInstances")
    def min_ready_instances(self) -> Optional[pulumi.Input[int]]:
        """
        The min ready instances.
        """
        return pulumi.get(self, "min_ready_instances")

    @min_ready_instances.setter
    def min_ready_instances(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "min_ready_instances", value)

    @property
    @pulumi.getter(name="scalingRuleEnable")
    def scaling_rule_enable(self) -> Optional[pulumi.Input[bool]]:
        """
        True whether the auto scaling policy is enabled. The value description is as follows: true: enabled state. false: disabled status. Valid values: `false`, `true`.
        """
        return pulumi.get(self, "scaling_rule_enable")

    @scaling_rule_enable.setter
    def scaling_rule_enable(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "scaling_rule_enable", value)

    @property
    @pulumi.getter(name="scalingRuleMetric")
    def scaling_rule_metric(self) -> Optional[pulumi.Input['ApplicationScalingRuleScalingRuleMetricArgs']]:
        """
        Monitor the configuration of the indicator elasticity strategy. See `scaling_rule_metric` below.
        """
        return pulumi.get(self, "scaling_rule_metric")

    @scaling_rule_metric.setter
    def scaling_rule_metric(self, value: Optional[pulumi.Input['ApplicationScalingRuleScalingRuleMetricArgs']]):
        pulumi.set(self, "scaling_rule_metric", value)

    @property
    @pulumi.getter(name="scalingRuleTimer")
    def scaling_rule_timer(self) -> Optional[pulumi.Input['ApplicationScalingRuleScalingRuleTimerArgs']]:
        """
        Configuration of Timing Resilient Policies. See `scaling_rule_timer` below.
        """
        return pulumi.get(self, "scaling_rule_timer")

    @scaling_rule_timer.setter
    def scaling_rule_timer(self, value: Optional[pulumi.Input['ApplicationScalingRuleScalingRuleTimerArgs']]):
        pulumi.set(self, "scaling_rule_timer", value)


@pulumi.input_type
class _ApplicationScalingRuleState:
    def __init__(__self__, *,
                 app_id: Optional[pulumi.Input[str]] = None,
                 min_ready_instance_ratio: Optional[pulumi.Input[int]] = None,
                 min_ready_instances: Optional[pulumi.Input[int]] = None,
                 scaling_rule_enable: Optional[pulumi.Input[bool]] = None,
                 scaling_rule_metric: Optional[pulumi.Input['ApplicationScalingRuleScalingRuleMetricArgs']] = None,
                 scaling_rule_name: Optional[pulumi.Input[str]] = None,
                 scaling_rule_timer: Optional[pulumi.Input['ApplicationScalingRuleScalingRuleTimerArgs']] = None,
                 scaling_rule_type: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering ApplicationScalingRule resources.
        :param pulumi.Input[str] app_id: Application ID.
        :param pulumi.Input[int] min_ready_instance_ratio: The min ready instance ratio.
        :param pulumi.Input[int] min_ready_instances: The min ready instances.
        :param pulumi.Input[bool] scaling_rule_enable: True whether the auto scaling policy is enabled. The value description is as follows: true: enabled state. false: disabled status. Valid values: `false`, `true`.
        :param pulumi.Input['ApplicationScalingRuleScalingRuleMetricArgs'] scaling_rule_metric: Monitor the configuration of the indicator elasticity strategy. See `scaling_rule_metric` below.
        :param pulumi.Input[str] scaling_rule_name: The name of a custom elastic scaling policy. In the application, the policy name cannot be repeated. It must start with a lowercase letter, and can only contain lowercase letters, numbers, and dashes (-), and no more than 32 characters. After the scaling policy is successfully created, the policy name cannot be modified.
        :param pulumi.Input['ApplicationScalingRuleScalingRuleTimerArgs'] scaling_rule_timer: Configuration of Timing Resilient Policies. See `scaling_rule_timer` below.
        :param pulumi.Input[str] scaling_rule_type: Flexible strategy type. Valid values: `mix`, `timing` and `metric`.
        """
        if app_id is not None:
            pulumi.set(__self__, "app_id", app_id)
        if min_ready_instance_ratio is not None:
            pulumi.set(__self__, "min_ready_instance_ratio", min_ready_instance_ratio)
        if min_ready_instances is not None:
            pulumi.set(__self__, "min_ready_instances", min_ready_instances)
        if scaling_rule_enable is not None:
            pulumi.set(__self__, "scaling_rule_enable", scaling_rule_enable)
        if scaling_rule_metric is not None:
            pulumi.set(__self__, "scaling_rule_metric", scaling_rule_metric)
        if scaling_rule_name is not None:
            pulumi.set(__self__, "scaling_rule_name", scaling_rule_name)
        if scaling_rule_timer is not None:
            pulumi.set(__self__, "scaling_rule_timer", scaling_rule_timer)
        if scaling_rule_type is not None:
            pulumi.set(__self__, "scaling_rule_type", scaling_rule_type)

    @property
    @pulumi.getter(name="appId")
    def app_id(self) -> Optional[pulumi.Input[str]]:
        """
        Application ID.
        """
        return pulumi.get(self, "app_id")

    @app_id.setter
    def app_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "app_id", value)

    @property
    @pulumi.getter(name="minReadyInstanceRatio")
    def min_ready_instance_ratio(self) -> Optional[pulumi.Input[int]]:
        """
        The min ready instance ratio.
        """
        return pulumi.get(self, "min_ready_instance_ratio")

    @min_ready_instance_ratio.setter
    def min_ready_instance_ratio(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "min_ready_instance_ratio", value)

    @property
    @pulumi.getter(name="minReadyInstances")
    def min_ready_instances(self) -> Optional[pulumi.Input[int]]:
        """
        The min ready instances.
        """
        return pulumi.get(self, "min_ready_instances")

    @min_ready_instances.setter
    def min_ready_instances(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "min_ready_instances", value)

    @property
    @pulumi.getter(name="scalingRuleEnable")
    def scaling_rule_enable(self) -> Optional[pulumi.Input[bool]]:
        """
        True whether the auto scaling policy is enabled. The value description is as follows: true: enabled state. false: disabled status. Valid values: `false`, `true`.
        """
        return pulumi.get(self, "scaling_rule_enable")

    @scaling_rule_enable.setter
    def scaling_rule_enable(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "scaling_rule_enable", value)

    @property
    @pulumi.getter(name="scalingRuleMetric")
    def scaling_rule_metric(self) -> Optional[pulumi.Input['ApplicationScalingRuleScalingRuleMetricArgs']]:
        """
        Monitor the configuration of the indicator elasticity strategy. See `scaling_rule_metric` below.
        """
        return pulumi.get(self, "scaling_rule_metric")

    @scaling_rule_metric.setter
    def scaling_rule_metric(self, value: Optional[pulumi.Input['ApplicationScalingRuleScalingRuleMetricArgs']]):
        pulumi.set(self, "scaling_rule_metric", value)

    @property
    @pulumi.getter(name="scalingRuleName")
    def scaling_rule_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of a custom elastic scaling policy. In the application, the policy name cannot be repeated. It must start with a lowercase letter, and can only contain lowercase letters, numbers, and dashes (-), and no more than 32 characters. After the scaling policy is successfully created, the policy name cannot be modified.
        """
        return pulumi.get(self, "scaling_rule_name")

    @scaling_rule_name.setter
    def scaling_rule_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "scaling_rule_name", value)

    @property
    @pulumi.getter(name="scalingRuleTimer")
    def scaling_rule_timer(self) -> Optional[pulumi.Input['ApplicationScalingRuleScalingRuleTimerArgs']]:
        """
        Configuration of Timing Resilient Policies. See `scaling_rule_timer` below.
        """
        return pulumi.get(self, "scaling_rule_timer")

    @scaling_rule_timer.setter
    def scaling_rule_timer(self, value: Optional[pulumi.Input['ApplicationScalingRuleScalingRuleTimerArgs']]):
        pulumi.set(self, "scaling_rule_timer", value)

    @property
    @pulumi.getter(name="scalingRuleType")
    def scaling_rule_type(self) -> Optional[pulumi.Input[str]]:
        """
        Flexible strategy type. Valid values: `mix`, `timing` and `metric`.
        """
        return pulumi.get(self, "scaling_rule_type")

    @scaling_rule_type.setter
    def scaling_rule_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "scaling_rule_type", value)


class ApplicationScalingRule(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 app_id: Optional[pulumi.Input[str]] = None,
                 min_ready_instance_ratio: Optional[pulumi.Input[int]] = None,
                 min_ready_instances: Optional[pulumi.Input[int]] = None,
                 scaling_rule_enable: Optional[pulumi.Input[bool]] = None,
                 scaling_rule_metric: Optional[pulumi.Input[Union['ApplicationScalingRuleScalingRuleMetricArgs', 'ApplicationScalingRuleScalingRuleMetricArgsDict']]] = None,
                 scaling_rule_name: Optional[pulumi.Input[str]] = None,
                 scaling_rule_timer: Optional[pulumi.Input[Union['ApplicationScalingRuleScalingRuleTimerArgs', 'ApplicationScalingRuleScalingRuleTimerArgsDict']]] = None,
                 scaling_rule_type: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Provides a Serverless App Engine (SAE) Application Scaling Rule resource.

        For information about Serverless App Engine (SAE) Application Scaling Rule and how to use it, see [What is Application Scaling Rule](https://www.alibabacloud.com/help/en/sae/latest/create-application-scaling-rule).

        > **NOTE:** Available since v1.159.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud
        import pulumi_random as random

        config = pulumi.Config()
        name = config.get("name")
        if name is None:
            name = "tf-example"
        default = alicloud.get_regions(current=True)
        default_integer = random.index.Integer("default",
            max=99999,
            min=10000)
        default_get_zones = alicloud.get_zones(available_resource_creation="VSwitch")
        default_network = alicloud.vpc.Network("default",
            vpc_name=name,
            cidr_block="10.4.0.0/16")
        default_switch = alicloud.vpc.Switch("default",
            vswitch_name=name,
            cidr_block="10.4.0.0/24",
            vpc_id=default_network.id,
            zone_id=default_get_zones.zones[0].id)
        default_security_group = alicloud.ecs.SecurityGroup("default", vpc_id=default_network.id)
        default_namespace = alicloud.sae.Namespace("default",
            namespace_id=f"{default.regions[0].id}:example{default_integer['result']}",
            namespace_name=name,
            namespace_description=name,
            enable_micro_registration=False)
        default_application = alicloud.sae.Application("default",
            app_description=name,
            app_name=f"{name}-{default_integer['result']}",
            namespace_id=default_namespace.id,
            image_url=f"registry-vpc.{default.regions[0].id}.aliyuncs.com/sae-demo-image/consumer:1.0",
            package_type="Image",
            security_group_id=default_security_group.id,
            vpc_id=default_network.id,
            vswitch_id=default_switch.id,
            timezone="Asia/Beijing",
            replicas=5,
            cpu=500,
            memory=2048)
        default_application_scaling_rule = alicloud.sae.ApplicationScalingRule("default",
            app_id=default_application.id,
            scaling_rule_name=name,
            scaling_rule_enable=True,
            scaling_rule_type="mix",
            min_ready_instances=3,
            min_ready_instance_ratio=-1,
            scaling_rule_timer={
                "period": "* * *",
                "schedules": [
                    {
                        "at_time": "08:00",
                        "max_replicas": 10,
                        "min_replicas": 3,
                    },
                    {
                        "at_time": "20:00",
                        "max_replicas": 50,
                        "min_replicas": 3,
                    },
                ],
            },
            scaling_rule_metric={
                "max_replicas": 50,
                "min_replicas": 3,
                "metrics": [
                    {
                        "metric_type": "CPU",
                        "metric_target_average_utilization": 20,
                    },
                    {
                        "metric_type": "MEMORY",
                        "metric_target_average_utilization": 30,
                    },
                    {
                        "metric_type": "tcpActiveConn",
                        "metric_target_average_utilization": 20,
                    },
                ],
                "scale_up_rules": {
                    "step": 10,
                    "disabled": False,
                    "stabilization_window_seconds": 0,
                },
                "scale_down_rules": {
                    "step": 10,
                    "disabled": False,
                    "stabilization_window_seconds": 10,
                },
            })
        ```

        ## Import

        Serverless App Engine (SAE) Application Scaling Rule can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:sae/applicationScalingRule:ApplicationScalingRule example <app_id>:<scaling_rule_name>
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] app_id: Application ID.
        :param pulumi.Input[int] min_ready_instance_ratio: The min ready instance ratio.
        :param pulumi.Input[int] min_ready_instances: The min ready instances.
        :param pulumi.Input[bool] scaling_rule_enable: True whether the auto scaling policy is enabled. The value description is as follows: true: enabled state. false: disabled status. Valid values: `false`, `true`.
        :param pulumi.Input[Union['ApplicationScalingRuleScalingRuleMetricArgs', 'ApplicationScalingRuleScalingRuleMetricArgsDict']] scaling_rule_metric: Monitor the configuration of the indicator elasticity strategy. See `scaling_rule_metric` below.
        :param pulumi.Input[str] scaling_rule_name: The name of a custom elastic scaling policy. In the application, the policy name cannot be repeated. It must start with a lowercase letter, and can only contain lowercase letters, numbers, and dashes (-), and no more than 32 characters. After the scaling policy is successfully created, the policy name cannot be modified.
        :param pulumi.Input[Union['ApplicationScalingRuleScalingRuleTimerArgs', 'ApplicationScalingRuleScalingRuleTimerArgsDict']] scaling_rule_timer: Configuration of Timing Resilient Policies. See `scaling_rule_timer` below.
        :param pulumi.Input[str] scaling_rule_type: Flexible strategy type. Valid values: `mix`, `timing` and `metric`.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ApplicationScalingRuleArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a Serverless App Engine (SAE) Application Scaling Rule resource.

        For information about Serverless App Engine (SAE) Application Scaling Rule and how to use it, see [What is Application Scaling Rule](https://www.alibabacloud.com/help/en/sae/latest/create-application-scaling-rule).

        > **NOTE:** Available since v1.159.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud
        import pulumi_random as random

        config = pulumi.Config()
        name = config.get("name")
        if name is None:
            name = "tf-example"
        default = alicloud.get_regions(current=True)
        default_integer = random.index.Integer("default",
            max=99999,
            min=10000)
        default_get_zones = alicloud.get_zones(available_resource_creation="VSwitch")
        default_network = alicloud.vpc.Network("default",
            vpc_name=name,
            cidr_block="10.4.0.0/16")
        default_switch = alicloud.vpc.Switch("default",
            vswitch_name=name,
            cidr_block="10.4.0.0/24",
            vpc_id=default_network.id,
            zone_id=default_get_zones.zones[0].id)
        default_security_group = alicloud.ecs.SecurityGroup("default", vpc_id=default_network.id)
        default_namespace = alicloud.sae.Namespace("default",
            namespace_id=f"{default.regions[0].id}:example{default_integer['result']}",
            namespace_name=name,
            namespace_description=name,
            enable_micro_registration=False)
        default_application = alicloud.sae.Application("default",
            app_description=name,
            app_name=f"{name}-{default_integer['result']}",
            namespace_id=default_namespace.id,
            image_url=f"registry-vpc.{default.regions[0].id}.aliyuncs.com/sae-demo-image/consumer:1.0",
            package_type="Image",
            security_group_id=default_security_group.id,
            vpc_id=default_network.id,
            vswitch_id=default_switch.id,
            timezone="Asia/Beijing",
            replicas=5,
            cpu=500,
            memory=2048)
        default_application_scaling_rule = alicloud.sae.ApplicationScalingRule("default",
            app_id=default_application.id,
            scaling_rule_name=name,
            scaling_rule_enable=True,
            scaling_rule_type="mix",
            min_ready_instances=3,
            min_ready_instance_ratio=-1,
            scaling_rule_timer={
                "period": "* * *",
                "schedules": [
                    {
                        "at_time": "08:00",
                        "max_replicas": 10,
                        "min_replicas": 3,
                    },
                    {
                        "at_time": "20:00",
                        "max_replicas": 50,
                        "min_replicas": 3,
                    },
                ],
            },
            scaling_rule_metric={
                "max_replicas": 50,
                "min_replicas": 3,
                "metrics": [
                    {
                        "metric_type": "CPU",
                        "metric_target_average_utilization": 20,
                    },
                    {
                        "metric_type": "MEMORY",
                        "metric_target_average_utilization": 30,
                    },
                    {
                        "metric_type": "tcpActiveConn",
                        "metric_target_average_utilization": 20,
                    },
                ],
                "scale_up_rules": {
                    "step": 10,
                    "disabled": False,
                    "stabilization_window_seconds": 0,
                },
                "scale_down_rules": {
                    "step": 10,
                    "disabled": False,
                    "stabilization_window_seconds": 10,
                },
            })
        ```

        ## Import

        Serverless App Engine (SAE) Application Scaling Rule can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:sae/applicationScalingRule:ApplicationScalingRule example <app_id>:<scaling_rule_name>
        ```

        :param str resource_name: The name of the resource.
        :param ApplicationScalingRuleArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ApplicationScalingRuleArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 app_id: Optional[pulumi.Input[str]] = None,
                 min_ready_instance_ratio: Optional[pulumi.Input[int]] = None,
                 min_ready_instances: Optional[pulumi.Input[int]] = None,
                 scaling_rule_enable: Optional[pulumi.Input[bool]] = None,
                 scaling_rule_metric: Optional[pulumi.Input[Union['ApplicationScalingRuleScalingRuleMetricArgs', 'ApplicationScalingRuleScalingRuleMetricArgsDict']]] = None,
                 scaling_rule_name: Optional[pulumi.Input[str]] = None,
                 scaling_rule_timer: Optional[pulumi.Input[Union['ApplicationScalingRuleScalingRuleTimerArgs', 'ApplicationScalingRuleScalingRuleTimerArgsDict']]] = None,
                 scaling_rule_type: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ApplicationScalingRuleArgs.__new__(ApplicationScalingRuleArgs)

            if app_id is None and not opts.urn:
                raise TypeError("Missing required property 'app_id'")
            __props__.__dict__["app_id"] = app_id
            __props__.__dict__["min_ready_instance_ratio"] = min_ready_instance_ratio
            __props__.__dict__["min_ready_instances"] = min_ready_instances
            __props__.__dict__["scaling_rule_enable"] = scaling_rule_enable
            __props__.__dict__["scaling_rule_metric"] = scaling_rule_metric
            if scaling_rule_name is None and not opts.urn:
                raise TypeError("Missing required property 'scaling_rule_name'")
            __props__.__dict__["scaling_rule_name"] = scaling_rule_name
            __props__.__dict__["scaling_rule_timer"] = scaling_rule_timer
            if scaling_rule_type is None and not opts.urn:
                raise TypeError("Missing required property 'scaling_rule_type'")
            __props__.__dict__["scaling_rule_type"] = scaling_rule_type
        super(ApplicationScalingRule, __self__).__init__(
            'alicloud:sae/applicationScalingRule:ApplicationScalingRule',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            app_id: Optional[pulumi.Input[str]] = None,
            min_ready_instance_ratio: Optional[pulumi.Input[int]] = None,
            min_ready_instances: Optional[pulumi.Input[int]] = None,
            scaling_rule_enable: Optional[pulumi.Input[bool]] = None,
            scaling_rule_metric: Optional[pulumi.Input[Union['ApplicationScalingRuleScalingRuleMetricArgs', 'ApplicationScalingRuleScalingRuleMetricArgsDict']]] = None,
            scaling_rule_name: Optional[pulumi.Input[str]] = None,
            scaling_rule_timer: Optional[pulumi.Input[Union['ApplicationScalingRuleScalingRuleTimerArgs', 'ApplicationScalingRuleScalingRuleTimerArgsDict']]] = None,
            scaling_rule_type: Optional[pulumi.Input[str]] = None) -> 'ApplicationScalingRule':
        """
        Get an existing ApplicationScalingRule resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] app_id: Application ID.
        :param pulumi.Input[int] min_ready_instance_ratio: The min ready instance ratio.
        :param pulumi.Input[int] min_ready_instances: The min ready instances.
        :param pulumi.Input[bool] scaling_rule_enable: True whether the auto scaling policy is enabled. The value description is as follows: true: enabled state. false: disabled status. Valid values: `false`, `true`.
        :param pulumi.Input[Union['ApplicationScalingRuleScalingRuleMetricArgs', 'ApplicationScalingRuleScalingRuleMetricArgsDict']] scaling_rule_metric: Monitor the configuration of the indicator elasticity strategy. See `scaling_rule_metric` below.
        :param pulumi.Input[str] scaling_rule_name: The name of a custom elastic scaling policy. In the application, the policy name cannot be repeated. It must start with a lowercase letter, and can only contain lowercase letters, numbers, and dashes (-), and no more than 32 characters. After the scaling policy is successfully created, the policy name cannot be modified.
        :param pulumi.Input[Union['ApplicationScalingRuleScalingRuleTimerArgs', 'ApplicationScalingRuleScalingRuleTimerArgsDict']] scaling_rule_timer: Configuration of Timing Resilient Policies. See `scaling_rule_timer` below.
        :param pulumi.Input[str] scaling_rule_type: Flexible strategy type. Valid values: `mix`, `timing` and `metric`.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ApplicationScalingRuleState.__new__(_ApplicationScalingRuleState)

        __props__.__dict__["app_id"] = app_id
        __props__.__dict__["min_ready_instance_ratio"] = min_ready_instance_ratio
        __props__.__dict__["min_ready_instances"] = min_ready_instances
        __props__.__dict__["scaling_rule_enable"] = scaling_rule_enable
        __props__.__dict__["scaling_rule_metric"] = scaling_rule_metric
        __props__.__dict__["scaling_rule_name"] = scaling_rule_name
        __props__.__dict__["scaling_rule_timer"] = scaling_rule_timer
        __props__.__dict__["scaling_rule_type"] = scaling_rule_type
        return ApplicationScalingRule(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="appId")
    def app_id(self) -> pulumi.Output[str]:
        """
        Application ID.
        """
        return pulumi.get(self, "app_id")

    @property
    @pulumi.getter(name="minReadyInstanceRatio")
    def min_ready_instance_ratio(self) -> pulumi.Output[Optional[int]]:
        """
        The min ready instance ratio.
        """
        return pulumi.get(self, "min_ready_instance_ratio")

    @property
    @pulumi.getter(name="minReadyInstances")
    def min_ready_instances(self) -> pulumi.Output[Optional[int]]:
        """
        The min ready instances.
        """
        return pulumi.get(self, "min_ready_instances")

    @property
    @pulumi.getter(name="scalingRuleEnable")
    def scaling_rule_enable(self) -> pulumi.Output[bool]:
        """
        True whether the auto scaling policy is enabled. The value description is as follows: true: enabled state. false: disabled status. Valid values: `false`, `true`.
        """
        return pulumi.get(self, "scaling_rule_enable")

    @property
    @pulumi.getter(name="scalingRuleMetric")
    def scaling_rule_metric(self) -> pulumi.Output[Optional['outputs.ApplicationScalingRuleScalingRuleMetric']]:
        """
        Monitor the configuration of the indicator elasticity strategy. See `scaling_rule_metric` below.
        """
        return pulumi.get(self, "scaling_rule_metric")

    @property
    @pulumi.getter(name="scalingRuleName")
    def scaling_rule_name(self) -> pulumi.Output[str]:
        """
        The name of a custom elastic scaling policy. In the application, the policy name cannot be repeated. It must start with a lowercase letter, and can only contain lowercase letters, numbers, and dashes (-), and no more than 32 characters. After the scaling policy is successfully created, the policy name cannot be modified.
        """
        return pulumi.get(self, "scaling_rule_name")

    @property
    @pulumi.getter(name="scalingRuleTimer")
    def scaling_rule_timer(self) -> pulumi.Output[Optional['outputs.ApplicationScalingRuleScalingRuleTimer']]:
        """
        Configuration of Timing Resilient Policies. See `scaling_rule_timer` below.
        """
        return pulumi.get(self, "scaling_rule_timer")

    @property
    @pulumi.getter(name="scalingRuleType")
    def scaling_rule_type(self) -> pulumi.Output[str]:
        """
        Flexible strategy type. Valid values: `mix`, `timing` and `metric`.
        """
        return pulumi.get(self, "scaling_rule_type")

